/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "chatdock.h"
#include "ui_chatdock.h"
#include "config.h"


ChatDock::ChatDock(QWidget *parent) :
    QDockWidget(tr("Chat"), parent),
    ui(new Ui::ChatDock)
{

    ui->setupUi(this);
    connect(ui->sendButton, SIGNAL(clicked()), this, SLOT(sendClicked()));

}


ChatDock::~ChatDock()
{

    delete ui;

}


void ChatDock::appendChat(const QString &sender, const QString &text)
{

    if (Config::self()->isChatMessageEnabled(Config::PlayerChatType)) {
        ui->chatBrowser->append(QString("<%1> %2").arg(sender).arg(text));
    }

}


void ChatDock::addPlayer(const QString &nick, const int &id)
{

    Q_UNUSED(id);
    if (Config::self()->isChatMessageEnabled(Config::PlayerJoinedLeftType)) {
        appendChat(tr("Server"), tr("\"%1\" joined the game.").arg(nick));
    }

}


void ChatDock::removePlayer(const QString &nick, const int &id)
{

    Q_UNUSED(id);
    if (Config::self()->isChatMessageEnabled(Config::PlayerJoinedLeftType)) {
        appendChat(tr("Server"), tr("\"%1\" left the game.").arg(nick));
    }

}


void ChatDock::setRoll(const QString &player, const int &value)
{

    if (Config::self()->isChatMessageEnabled(Config::DiceRollType)) {
        appendChat(tr("Server"), tr("\"%1\" has rolled a %2.").arg(player).arg(QString::number(value)));
    }

}


void ChatDock::sendClicked()
{

    const QString text = ui->chatLine->text();
    ui->chatLine->clear();

    if (!text.isEmpty()) {
        emit sendChat(text);
    }

}

