/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef CLIENT_H
#define CLIENT_H

// lib
#include <QFrustrationLib/layout.h>
#include <QFrustrationLib/protocol.h>

// Qt
#include <QtCore/QObject>
#include <QtNetwork/QAbstractSocket>


namespace QFC {


class Game;
class ClientPrivate;
class Client : public QObject
{
    Q_OBJECT
    friend class ClientPrivate;


public:
    explicit Client(QObject *parent = 0);
    ~Client();

    QString nick() const;
    QString hostName() const;
    quint16 port() const;
    bool isConnected() const;

    void abort();
    void connectToServer(const QString &hostName, const quint16 &port, const QString &nick);
    void disconnectFromServer();
    void setNick(const QString &nick);

    void waitForDisconnected(const int &msecs = 30000);

    void requestRoll(const int &gameID);
    void requestTurn(const int &from, const int &to, const int &gameID);
    void requestGameList();
    void createGame(const QString &name, const QString &key, const QF::Layout &layout);
    void joinGame(const int &gameID, const QString &key);
    void startGame(const int &gameID);
    void gameChat(const QString &text, const int &gameID);
    void leaveGame(const int &gameID);
    void gameLeft(QFC::Game *game);


private:
    ClientPrivate *d;


signals:
    void connected(QFC::Client *self);
    void disconnected(QFC::Client *self);
    void gameCreated(QFC::Client *self, QFC::Game *game);
    void gotGameList(const QHash<int, QString> &list);
    void gameOpen(const int &gameID, const QString &name);
    void gameClosed(const int &gameID, const QString &name);
    void gameJoined(QFC::Game *game);
    void error(QFC::Client *self, const QF::Protocol::Error &error);
    void socketError(QFC::Client *self, const QAbstractSocket::SocketError &error, const QString &errorString);
    void hostFound();
    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator);
    void stateChanged(const QAbstractSocket::SocketState &socketState);


};


} // namespace QFC


#endif // CLIENT_H
