/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "client_game.h"
#include "client.h"

// Qt
#include <QtCore/QStringList>
#include <QtCore/QDebug>


namespace QFC {


Game::Game(const int &id, const QString &name, const QString &key, const QF::Layout &layout,
           const bool &owner, QFC::Client *parent)
    : QObject(parent),
    m_id(id),
    m_name(name),
    m_key(key),
    m_layout(layout),
    m_owner(owner)
{

    blockSignals(true);

    m_currentPlayer = 0;
    m_client = parent;
    QHash<int, QString> list;
    list.insert(0, parent->nick());
    setPlayerList(list);
    m_hasWinner = false;

}


Game::~Game()
{

    qDeleteAll(m_stones);

}


int Game::gameId() const
{

    return m_id;

}


QString Game::name() const
{

    return m_name;

}


QString Game::key() const
{

    return m_key;

}


QF::Layout Game::layout() const
{

    return m_layout;

}


bool Game::owner() const
{

    return m_owner;

}


int Game::playerId(const QString &nick) const
{

    return m_playerList.key(nick);

}


QStringList Game::players() const
{

    return m_playerList.values();

}


QFC::Client *Game::client() const
{

    return m_client;

}


QHash<int, QString> Game::playerList() const
{

    return m_playerList;

}


QList<QF::Stone*> Game::stones() const
{

    return m_stones;

}


bool Game::hasWinner() const
{

    return m_hasWinner;

}


int Game::currentPlayer() const
{

    return m_currentPlayer;

}


void Game::setPlayerList(const QHash<int, QString> &playerList)
{

    m_playerList = playerList;

}


void Game::setPlayerJoined(const int &id, const QString &nick)
{

   // Q_ASSERT(!m_playerList.contains(id));
    m_playerList.insert(id, nick);

    emit playerJoined(nick, id);

}


void Game::setPlayerLeft(const int &id, const QString &nick)
{

    //Q_ASSERT(m_playerList.contains(id));
    m_playerList.remove(id);

    emit playerLeft(nick, id);

}


void Game::requestStart()
{

    m_client->startGame(gameId());

}


void Game::addStone(QF::Stone *stone)
{

    m_stones.append(stone);
    emit stoneAdded(stone);

}


void Game::setCurrentPlayer(const int &playerID)
{

    m_currentPlayer = playerID;
    emit currentPlayerChanged(playerID);

}


void Game::setRoll(const int &value)
{

    emit diceRolled(value);

}


void Game::setTurn(const QHash<int, int> &turn)
{

    emit gotTurn(turn);

}


void Game::sendTurn(const int &from, const int &to)
{

    client()->requestTurn(from, to, gameId());

}


void Game::roll()
{

    client()->requestRoll(gameId());

}


void Game::setStoneMoved(const int &from, const int &to)
{

    for (int i = 0; i<  m_stones.size(); i++) {
        QF::Stone *stone = m_stones.at(i);

        if (stone->pos() == from) {
            stone->setPos(to);
        }
    }

}


void Game::sendChat(const QString &text)
{

    client()->gameChat(text, gameId());

}


void Game::appendChat(const QString &sender, const QString &text)
{

    emit newChatText(sender, text);

}


void Game::setGameClosed()
{

    emit gameClosed();

}


void Game::setGameStarted()
{

    emit gameStarted();

}


void Game::setWinner(const int &playerID)
{

    m_hasWinner = true;
    emit winner(playerID);

}


void Game::leave()
{

    client()->leaveGame(gameId());

}


} // namespace QFC
