/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef CLIENT_GAME_H
#define CLIENT_GAME_H

// lib
#include <QFrustrationLib/layout.h>
#include <QFrustrationLib/stone.h>

// Qt
#include <QtCore/QObject>
#include <QtCore/QHash>

namespace QFC {


class Client;
class Game : public QObject
{
    Q_OBJECT


public:
    explicit Game(const int &id, const QString &name, const QString &key,
                  const QF::Layout &layout,
                  const bool &owner, QFC::Client *parent);
    ~Game();

    int gameId() const;
    QString name() const;
    QString key() const;
    QF::Layout layout() const;
    bool owner() const;
    int playerId(const QString &nick) const;
    QStringList players() const;
    QFC::Client *client() const;
    QHash<int, QString> playerList() const;
    QList<QF::Stone*> stones() const;
    bool hasWinner() const;
    int currentPlayer() const;

    void setPlayerList(const QHash<int, QString> &playerList);
    void setPlayerJoined(const int &id, const QString &nick);
    void setPlayerLeft(const int &id, const QString &nick);
    void requestStart();
    void addStone(QF::Stone *stone);
    void setCurrentPlayer(const int &playerID);
    void setRoll(const int &value);
    void setTurn(const QHash<int, int> &turn);
    void sendTurn(const int &from, const int &to);
    void roll();
    void setStoneMoved(const int &from, const int &to);
    void sendChat(const QString &text);
    void appendChat(const QString &sender, const QString &text);
    void setGameClosed();
    void setGameStarted();
    void setWinner(const int &playerID);
    void leave();


private:
    int m_id;
    QString m_name;
    QString m_key;
    QF::Layout m_layout;
    bool m_owner;
    QHash<int, QString> m_playerList;
    QFC::Client *m_client;
    QList<QF::Stone*> m_stones;
    bool m_hasWinner;
    int m_currentPlayer;


signals:
    void playerJoined(const QString &nick, const int &id);
    void playerLeft(const QString &nick, const int &id);
    void stoneAdded(QF::Stone *stone);
    void currentPlayerChanged(const int &playerID);
    void diceRolled(const int &value);
    void gotTurn(const QHash<int, int> &turn);
    void newChatText(const QString &sender, const QString &text);
    void gameClosed();
    void gameStarted();
    void winner(const int &playerID);


};


} // namespace QFC


#endif // CLIENT_GAME_H
