/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "colorcombo.h"

// Qt
#include <QtGui/QPixmap>
#include <QtGui/QPainter>
#include <QtGui/QStyledItemDelegate>


#define ColorRole Qt::UserRole
class ColorDelegate : public QStyledItemDelegate
{


public:
    ColorDelegate(QObject *parent = 0) : QStyledItemDelegate(parent) {}
    ~ColorDelegate() {}

    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
    {

        return QSize(64, 32);

    }


    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {

        painter->save();

        QRect rect = option.rect.adjusted(2, 2, -2, -2);

        QColor color = index.data(ColorRole).value<QColor>();
        painter->fillRect(rect, color);


        if (option.state & QStyle::State_MouseOver) {
            painter->setOpacity(0.5);
            painter->fillRect(option.rect, option.palette.color(QPalette::Highlight));
        }

        painter->restore();

    }


};



ColorCombo::ColorCombo(QWidget *parent)
    : QComboBox(parent)
{

    setMinimumSize(80, 20);
    setIconSize(QSize(80, 16));
    setItemDelegate(new ColorDelegate(this));

    addColor(Qt::red);
    addColor(Qt::black);
    addColor(Qt::green);
    addColor(Qt::blue);
    addColor(Qt::yellow);
    addColor(Qt::cyan);
    addColor(Qt::magenta);
    addColor(Qt::white);

    connect(this, SIGNAL(currentIndexChanged(int)), this, SLOT(itemChanged(int)));

}


ColorCombo::~ColorCombo()
{


}


QColor ColorCombo::currentColor() const
{

    return itemData(currentIndex(), ColorRole).value<QColor>();

}


void ColorCombo::addColor(const QColor &color)
{

    QPixmap pixmap(80, 16);
    pixmap.fill(color);

    addItem(QIcon(pixmap), QString(), color);

}


void ColorCombo::addColor(const Qt::GlobalColor &color)
{

    addColor(QColor(color));

}


void ColorCombo::setCurrentColor(const QColor &color)
{

    for (int i = 0; i < count(); i++) {
        const QColor c = itemData(i, ColorRole).value<QColor>();
        if (c == color) {
            setCurrentIndex(i);
            break;
        }
    }

}


void ColorCombo::setCurrentColor(const Qt::GlobalColor &color)
{

    setCurrentColor(QColor(color));

}



void ColorCombo::itemChanged(const int &index)
{

    emit colorChanged(itemData(index, ColorRole).value<QColor>());

}


