/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef COMPUTERENEMY_H
#define COMPUTERENEMY_H


// own
#include "client.h"

// Qt
#include <QtCore/QObject>


class ComputerEnemy : public QObject
{
    Q_OBJECT


public:
    enum Difficulty {
        VeryEasy = 0,
        Easy = 1,
        Normal = 3,
        Hard = 4,
        VeryHard = 5
    };

    explicit ComputerEnemy(QObject *parent = 0);
    ~ComputerEnemy();

    void connectToServer(const QString &server, const quint16 &port);
    void joinGame(const int &gameID, const QString &key);
    void init(const QString &server, const quint16 &port, const int &gameID, const QString &key);
    void init(QFC::Game *game);


private:
    struct PlayerData {
        QList<int> homeIndexList;
        QList<int> targetIndexList;

        QList<QF::Stone*> inHome;
        QList<QF::Stone*> onWay;
        QList<QF::Stone*> inTarget;

        int start;
        int last;
    };

    QFC::Client *m_client;
    QString m_nick;
    QString m_server;
    quint16 m_port;
    int m_gameID;
    QFC::Game *m_game;
    QString m_key;
    QHash<int, int> m_turn;
    int m_delay;
    Difficulty m_difficulty;

    inline bool doIt(const int &percent) const;
    inline bool test(const Difficulty &d) const;
    inline QF::Stone *stoneOnIndex(const int &index) const;


private slots:
    void error(QFC::Client*, const QF::Protocol::Error &error);
    void gameCreated(QFC::Client *self, QFC::Game *game);
    void gotTurn(const QHash<int, int> &turn);
    void playerChanged(const int &playerID);
    void connected(QFC::Client*);
    void sendTurn();
    void requestRoll();
    void configChanged();


};


#endif // COMPUTERENEMY_H
