/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef CONFIG_H
#define CONFIG_H

// Qt
#include <QtCore/QObject>
#include <QtCore/QHash>
#include <QtCore/QPair>
#include <QtCore/QVariant>



class QSettings;
class SoundCore;
class Config : public QObject
{
    Q_OBJECT
    friend class ConfigSingleton;


public:
    enum KeyType {
        NextStoneKeyType = 0,
        RollKeyType = 1,
        SentKeyType = 2
    };

    enum ChatMessageType {
        PlayerChatType = 0,
        DiceRollType = 1,
        PlayerJoinedLeftType = 2
    };

    ~Config();

    static Config *self();
    SoundCore *sound() const;
    void setSound(SoundCore *sound);

    bool isWritable() const;
    QString configPath() const;
    int kiDelay() const;
    int animationSpeed() const;
    bool openGL() const;
    Qt::Key key(const Config::KeyType &type) const;
    bool isChatMessageEnabled(const Config::ChatMessageType &type) const;
    QVariant value(const QString &key) const;
    template < typename T >
    T value(const QString &key, const T &defaultValue)
    {

        if (!m_data.contains(key)) {
            return defaultValue;
        }

        QVariant value = m_data.value(key);
        if (value.canConvert<T>()) {
            return value.value<T>();
        }
        return defaultValue;

    }


    void setKIDelay(const int &delay);
    void setAnimationSpeed(const int &speed);
    void setOpenGL(const bool &enable);
    void setKey(const Config::KeyType &type, const Qt::Key &key);
    void setChatMessageEnabled(const Config::ChatMessageType &type, const bool &enabled);
    void setValue(const QString &key, const QVariant &value);
    void saveProxy();
    void loadProxy();

    void sendChanged();


private:
    explicit Config(QObject *parent = 0);

    QHash<QString, QVariant> m_data;
    SoundCore *m_sound;
    QSettings *m_settings;

    void load();
    void save();


private slots:
    void updateSounds();


signals:
    void configChanged();


};


#endif // CONFIG_H
