/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "generalconfigpage.h"
#include "ui_generalconfigpage.h"
#include "config.h"
#include "computerenemy.h"


GeneralConfigPage::GeneralConfigPage(QWidget *parent)
    : ConfigPage(parent),
    ui(new Ui::GeneralConfigPage)
{

    ui->setupUi(this);

#ifndef QT_QTOPENGL_FOUND
    ui->openGLCheck->hide();
#endif

    ui->speedCombo->addItem(tr("Very slow"), 2500);
    ui->speedCombo->addItem(tr("Slow"), 2000);
    ui->speedCombo->addItem(tr("Normal"), 1000);
    ui->speedCombo->addItem(tr("Fast"), 800);
    ui->speedCombo->addItem(tr("Very fast"), 500);
    ui->speedCombo->addItem(tr("Incredibly fast"), 100);

    ui->kiDifficultyCombo->addItem(tr("Very easy"), (int) ComputerEnemy::VeryEasy);
    ui->kiDifficultyCombo->addItem(tr("Easy"), (int) ComputerEnemy::Easy);
    ui->kiDifficultyCombo->addItem(tr("Normal"), (int) ComputerEnemy::Normal);
    ui->kiDifficultyCombo->addItem(tr("Hard"), (int) ComputerEnemy::Hard);
    ui->kiDifficultyCombo->addItem(tr("Very hard"), (int) ComputerEnemy::VeryHard);

    ui->animationSpeedCombo->addItem(tr("Very slow"), 700);
    ui->animationSpeedCombo->addItem(tr("Slow"), 500);
    ui->animationSpeedCombo->addItem(tr("Normal"), 350);
    ui->animationSpeedCombo->addItem(tr("Fast"), 150);
    ui->animationSpeedCombo->addItem(tr("Very fast"), 80);

}


GeneralConfigPage::~GeneralConfigPage()
{

    delete ui;

}


QString GeneralConfigPage::title() const
{

    return tr("General");

}


QString GeneralConfigPage::icon() const
{

    return "preferences-other";

}


void GeneralConfigPage::load()
{

#ifdef QT_QTOPENGL_FOUND
    ui->openGLCheck->setChecked(Config::self()->openGL());
#endif

    for (int i = 0; i < ui->speedCombo->count(); i++) {
        const int speed = ui->speedCombo->itemData(i).toInt();
        if (speed == Config::self()->kiDelay()) {
            ui->speedCombo->setCurrentIndex(i);
            break;
        }
    }

    for (int i = 0; i < ui->kiDifficultyCombo->count(); i++) {
        const int d = ui->kiDifficultyCombo->itemData(i).toInt();
        if (d == Config::self()->value("KI Difficulty").toInt()) {
            ui->kiDifficultyCombo->setCurrentIndex(i);
            break;
        }
    }

    for (int i = 0; i < ui->animationSpeedCombo->count(); i++) {
        const int speed = ui->animationSpeedCombo->itemData(i).toInt();
        if (speed == Config::self()->animationSpeed()) {
            ui->animationSpeedCombo->setCurrentIndex(i);
            break;
        }
    }

}


void GeneralConfigPage::save()
{

#ifdef QT_QTOPENGL_FOUND
    Config::self()->setOpenGL(ui->openGLCheck->isChecked());
#endif
    Config::self()->setKIDelay(ui->speedCombo->itemData(ui->speedCombo->currentIndex()).toInt());
    Config::self()->setValue("KI Difficulty", ui->kiDifficultyCombo->itemData(ui->kiDifficultyCombo->currentIndex()).toInt());
    Config::self()->setAnimationSpeed(ui->animationSpeedCombo->itemData(ui->animationSpeedCombo->currentIndex()).toInt());

}


void GeneralConfigPage::defaults()
{

#ifdef QT_QTOPENGL_FOUND
    ui->openGLCheck->setChecked(false);
#endif
    for (int i = 0; i < ui->speedCombo->count(); i++) {
        int speed = ui->speedCombo->itemData(i).toInt();
        if (speed == 1000) {
            ui->speedCombo->setCurrentIndex(i);
            break;
        }
    }

    for (int i = 0; i < ui->kiDifficultyCombo->count(); i++) {
        const int d = ui->kiDifficultyCombo->itemData(i).toInt();
        if (d == (int) ComputerEnemy::Normal) {
            ui->kiDifficultyCombo->setCurrentIndex(i);
            break;
        }
    }

    for (int i = 0; i < ui->animationSpeedCombo->count(); i++) {
        int speed = ui->animationSpeedCombo->itemData(i).toInt();
        if (speed == 350) {
            ui->animationSpeedCombo->setCurrentIndex(i);
            break;
        }
    }

}


void GeneralConfigPage::saveDefaults(Config *config)
{

#ifdef QT_QTOPENGL_FOUND
    config->setOpenGL(false);
#endif
    config->setKIDelay(1000);
    config->setAnimationSpeed(350);
    config->setValue("KI Difficulty", (int) ComputerEnemy::Normal);

}
