/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "keysconfigpage.h"
#include "ui_keysconfigpage.h"
#include "config.h"


KeysConfigPage::KeysConfigPage(QWidget *parent)
    : ConfigPage(parent),
    ui(new Ui::KeysConfigPage)
{

    ui->setupUi(this);
    connect(ui->nextStoneButton, SIGNAL(keyRequested(KeyButton*,Qt::Key)), this, SLOT(keyRequested(KeyButton*,Qt::Key)));
    connect(ui->rollButton, SIGNAL(keyRequested(KeyButton*,Qt::Key)), this, SLOT(keyRequested(KeyButton*,Qt::Key)));
    connect(ui->confirmButton, SIGNAL(keyRequested(KeyButton*,Qt::Key)), this, SLOT(keyRequested(KeyButton*,Qt::Key)));

}


KeysConfigPage::~KeysConfigPage()
{

    delete ui;

}


QString KeysConfigPage::title() const
{

    return tr("Control");

}


QString KeysConfigPage::icon() const
{

    return "input-keyboard";

}


void KeysConfigPage::load()
{

    ui->nextStoneButton->setKey(Config::self()->key(Config::NextStoneKeyType));
    ui->rollButton->setKey(Config::self()->key(Config::RollKeyType));
    ui->confirmButton->setKey(Config::self()->key(Config::SentKeyType));
    ui->singleClickCheck->setChecked(Config::self()->value("View-SingleClick", false));

}


void KeysConfigPage::save()
{

    Config::self()->setKey(Config::NextStoneKeyType, ui->nextStoneButton->key());
    Config::self()->setKey(Config::RollKeyType, ui->rollButton->key());
    Config::self()->setKey(Config::SentKeyType, ui->confirmButton->key());
    Config::self()->setValue("View-SingleClick", ui->singleClickCheck->isChecked());

}


void KeysConfigPage::defaults()
{

    ui->nextStoneButton->setKey(Qt::Key_N);
    ui->rollButton->setKey(Qt::Key_Space);
    ui->confirmButton->setKey(Qt::Key_Return);
    ui->singleClickCheck->setChecked(false);

}


void KeysConfigPage::saveDefaults(Config *config)
{

    config->setKey(Config::NextStoneKeyType, Qt::Key_N);
    config->setKey(Config::RollKeyType, Qt::Key_Space);
    config->setKey(Config::SentKeyType, Qt::Key_Return);
    config->setValue("View-SingleClick", false);

}


void KeysConfigPage::keyRequested(KeyButton *self, const Qt::Key &key)
{

    QList<KeyButton*> buttons;
    buttons << ui->nextStoneButton;
    buttons << ui->rollButton;
    buttons << ui->confirmButton;

    buttons.removeAll(self);
    foreach (KeyButton *button, buttons) {
        if (button->key() == key) {
            self->reset();
            return;
        }
    }
    self->setKey(key);

}

