/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "soundconfigpage.h"
#include "ui_soundconfigpage.h"
#include "config.h"

// lib
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtCore/QVariant>
#include <QtCore/QDir>


SoundConfigPage::SoundConfigPage(QWidget *parent)
    : ConfigPage(parent),
    ui(new Ui::SoundConfigPage)
{

    ui->setupUi(this);

}


SoundConfigPage::~SoundConfigPage()
{

    delete ui;

}


QString SoundConfigPage::title() const
{

    return tr("Sound");

}


QString SoundConfigPage::icon() const
{

    return "sound";

}


void SoundConfigPage::load()
{

    ui->diceEdit->setFile(Config::self()->value("Sound-Dice").toString());
    ui->winnerEdit->setFile(Config::self()->value("Sound-Winner").toString());
    ui->loserEdit->setFile(Config::self()->value("Sound-Loser").toString());
    ui->joinedEdit->setFile(Config::self()->value("Sound-PlayerJoined").toString());
    ui->leftEdit->setFile(Config::self()->value("Sound-PlayerLeft").toString());
    ui->enabledCheck->setChecked(Config::self()->value("Sound-Enabled").toBool());

}


void SoundConfigPage::save()
{

    Config::self()->setValue("Sound-Dice", ui->diceEdit->file());
    Config::self()->setValue("Sound-Winner", ui->winnerEdit->file());
    Config::self()->setValue("Sound-Loser", ui->loserEdit->file());
    Config::self()->setValue("Sound-PlayerJoined", ui->joinedEdit->file());
    Config::self()->setValue("Sound-PlayerLeft", ui->leftEdit->file());
    Config::self()->setValue("Sound-Enabled", ui->enabledCheck->isChecked());

}


void SoundConfigPage::defaults()
{

    const QString path = FrustGlobal::instance()->findFrustDir("sounds");
    ui->diceEdit->setFile(path+QDir::separator()+"dice.ogg");
    ui->winnerEdit->setFile(path+QDir::separator()+"winner.ogg");
    ui->loserEdit->setFile(path+QDir::separator()+"loser.ogg");
    ui->joinedEdit->setFile(path+QDir::separator()+"playerjoined.ogg");
    ui->leftEdit->setFile(path+QDir::separator()+"playerleft.ogg");
    ui->enabledCheck->setChecked(true);

}


void SoundConfigPage::saveDefaults(Config *config)
{

    const QString path = FrustGlobal::instance()->findFrustDir("sounds");
    config->setValue("Sound-Dice", path+QDir::separator()+"dice.ogg");
    config->setValue("Sound-Winner", path+QDir::separator()+"winner.ogg");
    config->setValue("Sound-Loser", path+QDir::separator()+"loser.ogg");
    config->setValue("Sound-PlayerJoined", path+QDir::separator()+"playerjoined.ogg");
    config->setValue("Sound-PlayerLeft", path+QDir::separator()+"playerleft.ogg");
    config->setValue("Sound-Enabled", true);

}
