/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "themeconfigpage.h"
#include "ui_themeconfigpage.h"
#include "theme.h"
#include "config.h"

// Qt
#include <QtGui/QListWidgetItem>


ThemeConfigPage::ThemeConfigPage(QWidget *parent)
    : ConfigPage(parent),
    ui(new Ui::ThemeConfigPage)
{

    ui->setupUi(this);
    connect(ui->themeList, SIGNAL(itemSelectionChanged()), this, SLOT(themeSelectionChanged()));

}


ThemeConfigPage::~ThemeConfigPage()
{

    delete ui;

}


QString ThemeConfigPage::title() const
{

    return tr("Theme");

}


QString ThemeConfigPage::icon() const
{

    return "theme";

}


void ThemeConfigPage::load()
{

    QList<Theme::ThemeData> list = Theme::currentTheme()->availableThemes();
    foreach (const Theme::ThemeData &data, list) {
        QListWidgetItem *item = new QListWidgetItem;
        item->setText(data.name);
        item->setData(DescriptionRole, data.description);
        item->setData(AuthorRole, data.author);
        item->setData(ContactRole, data.email);
        item->setData(VersionRole, data.version);
        item->setData(PreviewRole, data.preview);

        ui->themeList->addItem(item);
        if (Theme::currentTheme()->currentThemeName() == data.name) {
            item->setSelected(true);
        }
    }
    themeSelectionChanged();

}


void ThemeConfigPage::save()
{

    QList<QListWidgetItem*> items = ui->themeList->selectedItems();
    if (items.isEmpty()) {
        return;
    }
    QListWidgetItem *item = items.at(0);
    Theme::currentTheme()->setCurrentTheme(item->data(NameRole).toString());

}


void ThemeConfigPage::defaults()
{

    for (int i = 0; i < ui->themeList->count(); i++) {
        QListWidgetItem *item = ui->themeList->item(i);
        if (item->text() == QLatin1String("Frusti")) {
            item->setSelected(true);
            break;
        }
    }

}


void ThemeConfigPage::saveDefaults(Config *config)
{

    config->setValue("Theme", "Frusti");

}


void ThemeConfigPage::themeSelectionChanged()
{

    QList<QListWidgetItem*> items = ui->themeList->selectedItems();
    if (items.isEmpty()) {
        return;
    }
    QListWidgetItem *item = items.at(0);

    ui->authorLabel->setText(item->data(AuthorRole).toString());
    ui->descriptionLabel->setText(item->data(DescriptionRole).toString());
    ui->versionLabel->setText(item->data(VersionRole).toString());
    ui->contactLabel->setText(QString("<a href=\"mailto:%1\">%1</a>").arg(item->data(ContactRole).toString()));

    QPixmap pixmap(item->data(PreviewRole).toString());
    pixmap = pixmap.scaled(150, 150, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    ui->previewLabel->setPixmap(pixmap);

}

