/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "desktopfile.h"

// Qt
#include <QtCore/QFile>
#include <QtCore/QDebug>


DesktopFile::DesktopFile(const QString &file)
    : m_file(file)
{


}


DesktopFile::~DesktopFile()
{

    QMutableHashIterator<QString, Section*> it(m_data);
    while (it.hasNext()) {
        it.next();
        Section *sec = it.value();
        it.remove();
        delete sec;
    }

}


QString DesktopFile::error() const
{

    return m_error;

}


bool DesktopFile::init()
{

    QFile file(m_file);
    if (!file.open(QIODevice::ReadOnly)) {
        m_error = file.errorString();
        return false;
    }

    Section *currentSection = 0;
    while (!file.atEnd()) {
        const QString line = file.readLine().trimmed();
        if (line.isEmpty()) {
            continue;
        }

        if (line.startsWith('[') && line.endsWith(']')) {
            QString name = line;
            name.remove(0, 1);
            name.remove(name.length()-1, 1);
            if (m_data.contains(name)) {
                currentSection = m_data.value(name);
            } else {
                currentSection = new Section;
                m_data.insert(name, currentSection);
            }
        } else if (currentSection) {
            int pos = line.indexOf('=', 0);
            if (pos == -1) {
                qDebug() << "DesktopFile: invalid line:" << line;
                continue;
            }
            const QString key = line.mid(0, pos);
            const QString value = line.mid(pos+1);

            currentSection->insert(key, value);
        } else {
            qDebug() << "no section for line:" << line;
        }
    }
    return true;

}


QString DesktopFile::value(const QString &section, const QString &key) const
{

    if (!m_data.contains(section)) {
        qDebug() << "no such section:" << section << "Sections:" << m_data.keys();
        return QString();
    }
    Section *sec = m_data.value(section);
    return sec->value(key);

}

