/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "configdialog.h"
#include "ui_configdialog.h"
#include "theme.h"
#include "config.h"
#include "keybutton.h"
#include "themeconfigpage.h"
#include "proxyconfigpage.h"
#include "keysconfigpage.h"
#include "generalconfigpage.h"
#include "chatconfigpage.h"
#include "soundconfigpage.h"

// lib
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtNetwork/QNetworkProxy>
#include <QtCore/QDebug>
#include <QtGui/QStackedLayout>
#include <QtGui/QApplication>


ConfigDialog::ConfigDialog(QWidget *parent)
    : QDialog(parent)
{

    setAttribute(Qt::WA_DeleteOnClose, true);
    setWindowIcon(FRUST_ICON("configure"));

    ui = new Ui::ConfigDialog;
    ui->setupUi(this);
    setWindowTitle(tr("Configure: %1").arg(qApp->applicationName()));
    ui->saveButton->setIcon(FRUST_ICON("dialog-ok"));
    ui->cancelButton->setIcon(FRUST_ICON("dialog-cancel"));
    ui->defaultsButton->setIcon(FRUST_ICON("document-revert"));

    QStackedLayout *layout = new QStackedLayout;
    ui->configWidget->setLayout(layout);
    connect(ui->configTabWidget, SIGNAL(tabRequested(QWidget*)), layout, SLOT(setCurrentWidget(QWidget*)));
    connect(ui->defaultsButton, SIGNAL(clicked()), this, SLOT(defaultsClicked()));

    QList<ConfigPage*> pageList;
    pageList << new GeneralConfigPage(this);
    pageList << new ThemeConfigPage(this);
    pageList << new ProxyConfigPage(this);
    pageList << new KeysConfigPage(this);
    pageList << new ChatConfigPage(this);
#ifdef HAVE_PHONON
    pageList << new SoundConfigPage(this);
#endif

    foreach (ConfigPage *page, pageList) {
        layout->addWidget(page);
        ui->configTabWidget->addTab(page);
        page->load();
    }
   // layout->setCurrentIndex(0);

}


ConfigDialog::~ConfigDialog()
{

    if (result() == QDialog::Accepted) {
        foreach (ConfigPage *page, ui->configTabWidget->tabs()) {
            page->save();
        }
        Config::self()->sendChanged();
    }

    delete ui;

}


void ConfigDialog::defaultsClicked()
{

    foreach (ConfigPage *page, ui->configTabWidget->tabs()) {
        page->defaults();
    }

}

