/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "connectingdialog.h"
#include "ui_connectingdialog.h"
#include "client.h"

// lib
#include <QFrustrationSharedGUI/global.h>


ConnectingDialog::ConnectingDialog(QFC::Client *client, QWidget *parent)
    : QDialog(parent),
    ui(new Ui::ConnectingDialog)
{

    setAttribute(Qt::WA_DeleteOnClose, true);
    ui->setupUi(this);
    ui->cancelButton->setIcon(FRUST_ICON("dialog-cancel"));
    setWindowIcon(FRUST_ICON("network-connect"));

    m_client = client;
    connect(client, SIGNAL(connected(QFC::Client*)), this, SLOT(connected()));
    connect(client, SIGNAL(disconnected(QFC::Client*)), this, SLOT(disconnected()));
    connect(client, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
            this, SLOT(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
    connect(client, SIGNAL(hostFound()), this, SLOT(hostFound()));
    connect(client, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SLOT(stateChanged(QAbstractSocket::SocketState)));
    connect(client, SIGNAL(error(QFC::Client*,QF::Protocol::Error)), this, SLOT(reject()));

    connect(this, SIGNAL(finished(int)), this, SLOT(dialogFinished(int)));

}


ConnectingDialog::~ConnectingDialog()
{

    delete ui;

}


void ConnectingDialog::start(QWidget *parent, QFC::Client *client)
{

    ConnectingDialog *dialog = new ConnectingDialog(client, parent);
    dialog->show();
    dialog->raise();

}


void ConnectingDialog::dialogFinished(const int &result)
{

    if (result != QDialog::Accepted) {
        m_client->abort();
    }

}


void ConnectingDialog::connected()
{

    accept();

}


void ConnectingDialog::disconnected()
{

    reject();

}


void ConnectingDialog::error(const QAbstractSocket::SocketError &socketError)
{

    Q_UNUSED(socketError);
    reject();

}


void ConnectingDialog::hostFound()
{

    setWindowTitle(tr("Connecting: %1").arg(m_client->hostName()));

}


void ConnectingDialog::proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator)
{

    // TODO: impl me
    Q_UNUSED(proxy);
    Q_UNUSED(authenticator);

}


void ConnectingDialog::stateChanged(const QAbstractSocket::SocketState &socketState)
{

    QString text;
    switch (socketState) {
    case QAbstractSocket::UnconnectedState: reject(); break;
    case QAbstractSocket::HostLookupState: text = tr("Performing a host name lookup."); break;
    case QAbstractSocket::ConnectingState: text = tr("Establishing connection..."); break;
    case QAbstractSocket::ConnectedState: text = tr("Connection established.\nPerforming login..."); break;
    case QAbstractSocket::BoundState: break;
    case QAbstractSocket::ClosingState: break;
    case QAbstractSocket::ListeningState: break;
    }

    if (!text.isEmpty()) {
        ui->log->append(text);
    }

}


