/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "creategamedialog.h"
#include "ui_creategamedialog.h"

// lib
#include <QFrustrationSharedGUI/global.h>
#include <QFrustrationSharedGUI/errordialog.h>

// Qt
#include <QtCore/QDebug>
#include <QtCore/QFile>


CreateGameDialog::CreateGameDialog(QWidget *parent)
    : QDialog(parent)
{

    setAttribute(Qt::WA_DeleteOnClose, true);
    setWindowIcon(FRUST_ICON("applications-games"));

    ui = new Ui::CreateGameDialog;
    ui->setupUi(this);
    ui->createButton->setIcon(FRUST_ICON("dialog-ok"));
    ui->cancelButton->setIcon(FRUST_ICON("dialog-cancel"));
    ui->fileRequester->setNameFilters(QStringList() << "Frust Layout (*.frust)");

    showKey();

    connect(this, SIGNAL(finished(int)), this, SLOT(dialogFinsihed(int)));
    connect(ui->showCheck, SIGNAL(clicked()), this, SLOT(showKey()));

    setMaximumHeight(minimumSizeHint().height());

}


CreateGameDialog::~CreateGameDialog()
{

    delete ui;

}


void CreateGameDialog::dialogFinsihed(const int &result)
{

    if (result == QDialog::Accepted) {
        const QString name = ui->nameEdit->text();
        const QString key = ui->keyEdit->text();

        if (!name.isEmpty()) {
            QF::Layout layout;
            if (ui->fourPlayerRadio->isChecked()) {
                layout.buildLayout(QF::Layout::FourPlayersLayout);
            } else if (ui->sixPlayerRadio->isChecked()) {
                layout.buildLayout(QF::Layout::SixPlayersLayout);
            } else {
                QFile file(ui->fileRequester->file());
                if (!file.open(QIODevice::ReadOnly)) {
                    ErrorDialog::showError(parentWidget(), tr("Error"), file.errorString());
                    return;
                }
                layout.load(file.readAll());
                file.close();
            }

            if (!layout.isValid()) {
                ErrorDialog::showError(parentWidget(), tr("Error"), tr("This layout is invalid."));
                return;
            }
            emit finished(name, key, layout);
        }
    }

}


void CreateGameDialog::showKey()
{

    if (ui->showCheck->isChecked()) {
        ui->keyEdit->setEchoMode(QLineEdit::Normal);
    } else {
        ui->keyEdit->setEchoMode(QLineEdit::Password);
    }

}


