/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "serverdialog.h"
#include "ui_serverdialog.h"
#include "config.h"

// lib
#include <QFrustrationSharedGUI/global.h>


ServerDialog::ServerDialog(const QString &server, const int &port, QWidget *parent)
    : QDialog(parent),
    ui(new Ui::ServerDialog)
{

    setAttribute(Qt::WA_DeleteOnClose, true);

    ui->setupUi(this);
    ui->serverEdit->setText(server);
    ui->portEdit->setValue(port);

    connect(this, SIGNAL(finished(int)), this, SLOT(dialogFinished(int)));

    setWindowIcon(FRUST_ICON("network-connect"));
    ui->connectButton->setIcon(FRUST_ICON("network-connect"));
    ui->cancelButton->setIcon(FRUST_ICON("dialog-cancel"));

    setMaximumHeight(minimumSizeHint().height());


    if (Config::self()->value("ConnectToServerDialog-Remember", false)) {
        ui->rememberCheck->setChecked(true);
        ui->nickLine->setText(Config::self()->value("ConnectToServerDialog-Nick", tr("Angry Player")));
    }

}


ServerDialog::~ServerDialog()
{

    Config::self()->setValue("ConnectToServerDialog-Remember", ui->rememberCheck->isChecked());
    if (ui->rememberCheck->isChecked()) {
        Config::self()->setValue("ConnectToServerDialog-Nick", ui->nickLine->text());
    }

    delete ui;

}


void ServerDialog::dialogFinished(const int &result)
{

    const QString nick = ui->nickLine->text();
    const QString server = ui->serverEdit->text();
    if (result == QDialog::Accepted && !server.isEmpty() && !nick.isEmpty()) {
        emit finished(server, ui->portEdit->value(), nick);
    }

}
