/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/



// own
#include "dicewidget.h"
#include "theme.h"
#include "config.h"
#include "soundcore.h"

// lib
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtGui/QPainter>
#include <QtCore/QTimer>
#include <QtGui/QGraphicsSceneMouseEvent>
#include <QtCore/QDateTime>
#include <QtGui/QCursor>
#include <QtGui/QGraphicsSceneHoverEvent>


DiceWidget::DiceWidget(QGraphicsWidget *parent)
    : QGraphicsWidget(parent)
{

    m_targetValue = 1;
    m_currentValue = 3;

    m_timer = new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), this, SLOT(timeout()));

    m_count = 0;

    setAcceptHoverEvents(true);

    qsrand(QDateTime::currentDateTime().toTime_t());

    connect(Config::self(), SIGNAL(configChanged()), this, SLOT(configChanged()));
    configChanged();

}



DiceWidget::~DiceWidget()
{

    delete m_timer;

}



void DiceWidget::roll(const int &value)
{

    Config::self()->sound()->play("Dice");
    if (m_timer->isActive()) {
        m_timer->stop();
    }

    m_targetValue = value;
    m_currentValue = 1;
    m_count = 0;

    m_timer->start(100);

}




void DiceWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{

    Q_UNUSED(option);
    Q_UNUSED(widget);

    Theme::currentTheme()->renderDice(painter, m_currentValue, rect());

}


void DiceWidget::timeout()
{

    ++m_count;

    if (m_count > 5) {
        m_currentValue = m_targetValue;
        m_timer->stop();
        m_count = 0;
        update();
        return;
    }
    int num = m_currentValue;
    while (num == m_currentValue) {
        num = (qrand() % 6) + 1;
    }
    m_currentValue = num;
    update();

}


void DiceWidget::configChanged()
{


}


void DiceWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{

    event->accept();
    setCursor(Qt::OpenHandCursor);

}


void DiceWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{

    event->accept();
    setCursor(QCursor());

}


void DiceWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    setCursor(Qt::ClosedHandCursor);

}


void DiceWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();

}


void DiceWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    if (isUnderMouse()) {
        setCursor(Qt::OpenHandCursor);
    } else {
        setCursor(QCursor());
    }
    emit clicked();

}


