/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/



// own
#include "fieldwidget.h"
#include "theme.h"
#include "config.h"

// Qt
#include <QtGui/QPainter>
#include <QtCore/QDebug>
#include <QtCore/QPropertyAnimation>
#include <QtGui/QGraphicsSceneHoverEvent>
#include <QtGui/QCursor>


FieldWidget::FieldWidget(const QColor &color, const int &id, QGraphicsWidget *parent)
    : QGraphicsWidget(parent),
    m_color(color),
    m_id(id),
    m_highlight(false)
{

    setHighlightColor(Qt::transparent);

    m_highlightAnimation = new QPropertyAnimation(this);
    m_highlightAnimation->setTargetObject(this);
    m_highlightAnimation->setPropertyName("scale");

    setZValue(1);

    setCacheMode(QGraphicsItem::DeviceCoordinateCache);
    setAcceptHoverEvents(true);

}


FieldWidget::~FieldWidget()
{

    delete m_highlightAnimation;

}


QColor FieldWidget::highlightColor() const
{

    return m_highlightColor;

}


void FieldWidget::setHighlightColor(const QColor &color)
{

    m_highlightColor = color;
    update();

}


int FieldWidget::id() const
{

    return m_id;

}


int FieldWidget::playerID() const
{

    return m_playerID;

}


QF::Layout::FieldType FieldWidget::fieldType() const
{

    return m_fieldType;

}


bool FieldWidget::isHighlighted() const
{

    return m_highlight;

}


void FieldWidget::setPlayerID(const int &id)
{

    m_playerID = id;

}


void FieldWidget::setFieldType(const QF::Layout::FieldType &type)
{

    m_fieldType = type;

}


void FieldWidget::setColor(const QColor &color)
{

    m_color = color;
    update();

}


void FieldWidget::setHighlighted(const bool &highlight)
{

    if (m_highlight == highlight) {
        return;
    }
    m_highlight = highlight;

    if (m_highlightAnimation->state() == QPropertyAnimation::Running) {
        m_highlightAnimation->stop();
    }

    if (highlight) {
        setZValue(zValue()+1);
        m_highlightAnimation->setDuration(Config::self()->animationSpeed()*2);
        m_highlightAnimation->setEasingCurve(QEasingCurve::SineCurve);
        m_highlightAnimation->setLoopCount(-1); // forever
        m_highlightAnimation->setStartValue(1.0);
        m_highlightAnimation->setEndValue(1.2);
    } else {
        setZValue(zValue()-1);
        m_highlightAnimation->setDuration(Config::self()->animationSpeed());
        m_highlightAnimation->setEasingCurve(QEasingCurve::Linear);
        m_highlightAnimation->setLoopCount(1);
        m_highlightAnimation->setStartValue(scale());
        m_highlightAnimation->setEndValue(1.0);
    }
    setTransformOriginPoint(contentsRect().center());
    m_highlightAnimation->start();

}


void FieldWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{

    Q_UNUSED(option);
    Q_UNUSED(widget);

    Theme::currentTheme()->renderField(painter, contentsRect(), playerID(), fieldType());

}


void FieldWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{

    event->accept();
    if (isHighlighted()) {
        setCursor(Qt::PointingHandCursor);
    }

}


void FieldWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{

    event->accept();
    if (isHighlighted()) {
        setCursor(QCursor());
    }

}


void FieldWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    Q_UNUSED(event);

}


void FieldWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{

    Q_UNUSED(event);

}


void FieldWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    Q_UNUSED(event);
    if (m_highlight) {
        emit clicked(this);
    }

}




