/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef FIELDWIDGET_H
#define FIELDWIDGET_H

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtGui/QGraphicsWidget>


class QPropertyAnimation;
class StoneWidget;
class FieldWidget : public QGraphicsWidget
{
    Q_OBJECT
    Q_PROPERTY(QColor highlightColor READ highlightColor WRITE setHighlightColor)


public:
    explicit FieldWidget(const QColor &color, const int &id, QGraphicsWidget *parent = 0);
    ~FieldWidget();

    QColor highlightColor() const;
    void setHighlightColor(const QColor &color);

    int id() const;

    int playerID() const;
    QF::Layout::FieldType fieldType() const;
    bool isHighlighted() const;

    void setPlayerID(const int &id);
    void setFieldType(const QF::Layout::FieldType &type);

    void setColor(const QColor &color);
    void setHighlighted(const bool &highlight);    
    StoneWidget *stone;


private:
    QColor m_color;
    int m_id;
    bool m_highlight;
    QColor m_highlightColor;
    int m_playerID;
    QF::Layout::FieldType m_fieldType;
    QPropertyAnimation *m_highlightAnimation;


protected:
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);

    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);


signals:
    void clicked(FieldWidget *self);


};


#endif // FIELDWIDGET_H
