/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef GAMETAB_H
#define GAMETAB_H


// own
#include "client_game.h"

// Qt
#include <QtGui/QMainWindow>


namespace Ui {
    class GameTab;
}
class ChatDock;
class PlayerListDock;
class ComputerEnemy;
class GameTab : public QMainWindow
{
    Q_OBJECT


public:
    explicit GameTab(QWidget *parent = 0);
    ~GameTab();

    QString title() const;

    void setGame(QFC::Game *game);
    void startQuickGame(const int &kiCount);


private:
    Ui::GameTab *ui;
    QWeakPointer<QFC::Game> m_game;
    ChatDock *m_chatDock;
    PlayerListDock *m_playerList;
    QList<QAction*> m_actions;
    int m_currentPlayer;
    ComputerEnemy *m_ki;

    QAction *getAction(const QString &name);


private slots:
    void init();
    void playerJoined(const QString &nick, const int &id);
    void playerLeft(const QString &nick, const int &id);
    void startClicked();
    void stoneAdded(QF::Stone *stone);
    void currentPlayerChanged(const int &newCurrentPlayerID);
    void diceRolled(const int &value);
    void gotTurn(const QHash<int, int> &turn);
    void diceClicked();
    void sendTurn(const int &from, const int &to);
    void newChatText(const QString &sender, const QString &text);
    void sendChat(const QString &text);
    void gameStarted();
    void winner(const int &playerID);
    void gameClosed();
    void zoomInClicked();
    void zoomOutClicked();
    void addComputerPlayerClicked();
    void fullViewTriggered(const bool &checked);


signals:
    void leaveGame(QWidget *self);
    void fullViewModeTriggered(GameTab *tab, const bool &enabled);


};


#endif // GAMETAB_H
