/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "localservertab.h"
#include "ui_localservertab.h"

// lib
#include <QFrustrationLib/server.h>
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtCore/QDebug>

// C
#include <math.h>


LocalServerTab::LocalServerTab(QWidget *parent)
    : QWidget(parent),
    ui(new Ui::LocalServerTab)
{

    m_server = 0;
    m_rec = 0;
    m_sent = 0;

    ui->setupUi(this);

    dataSent(0);
    receivingData(0);

}


LocalServerTab::~LocalServerTab()
{

    delete ui;
    if (m_server) {
        delete m_server;
    }

}


QFS::Server *LocalServerTab::server() const
{

    return m_server;

}


void LocalServerTab::setServer(QFS::Server *server, const bool &publicServer)
{

    m_server = server;
    m_server->setParent(this);

    ui->statusLog->append(tr("Starting Server..."));

    connect(m_server, SIGNAL(clientConnected(QString)), this, SLOT(clientConnected(QString)));
    connect(m_server, SIGNAL(clientDisconnected(QString)), this, SLOT(clientDisconnected(QString)));
    connect(m_server, SIGNAL(gameCreated(QString)), this, SLOT(gameCreated(QString)));
    connect(m_server, SIGNAL(gameStarted(QString)), this, SLOT(gameStarted(QString)));
    connect(m_server, SIGNAL(gameClosed(QString)), this, SLOT(gameClosed(QString)));
    connect(m_server, SIGNAL(receivingData(qint64)), this, SLOT(receivingData(qint64)));
    connect(m_server, SIGNAL(dataSent(qint64)), this, SLOT(dataSent(qint64)));

    if (!m_server->start(publicServer)) {
        ui->statusLog->append(tr("Failed!"));
        if (!m_server->errorString().isEmpty()) {
            ui->statusLog->append(tr("Error: %1").arg(m_server->errorString()));
        }

        delete m_server;
        m_server = 0;
        return;
    }

    ui->portLabel->setText(QString::number(m_server->port()));
    ui->addressLabel->setText(m_server->address());

    ui->statusLog->append(tr("Server Started!"));

}


void LocalServerTab::stop()
{

    if (!m_server) {
        return;
    }

    ui->statusLog->append(tr("Stopping Server..."));

    delete m_server;
    m_server = 0;

    m_rec = 0;
    m_sent = 0;

    ui->statusLog->append(tr("Server stopped."));

}


void LocalServerTab::clientConnected(const QString &nick)
{

    ui->statusLog->append(tr("Client connected: %1").arg(nick));

}


void LocalServerTab::clientDisconnected(const QString &nick)
{

    ui->statusLog->append(tr("Client disconnected: %1").arg(nick));

}


void LocalServerTab::gameCreated(const QString &name)
{

    ui->statusLog->append(tr("Game created: %1").arg(name));

}


void LocalServerTab::gameStarted(const QString &name)
{

    ui->statusLog->append(tr("Game started: %1").arg(name));

}


void LocalServerTab::gameClosed(const QString &name)
{

    ui->statusLog->append(tr("Game closed: %1").arg(name));

}


void LocalServerTab::receivingData(const qint64 &bytes)
{

    m_rec += bytes;
    ui->recLabel->setText(QString("%1 Mb").arg(locale().toString((float) (m_rec/1024.0)/1024.0, 'f', 3)));

}


void LocalServerTab::dataSent(const qint64 &bytes)
{

    m_sent += bytes;
    ui->sentLabel->setText(QString("%1 Mb").arg(locale().toString((float) (m_sent/1024.0)/1024.0, 'f', 3)));

}


