/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H

// own
#include "client.h"

// lib
#include <QFrustrationLib/layout.h>
#include <QFrustrationLib/stone.h>

// Qt
#include <QtGui/QMainWindow>

namespace QFC {
    class Client;
    class Game;
}

class QStackedLayout;
class GameTab;
class LocalServerTab;
namespace Ui {
    class MainWindow;
}
class ServerTabWidget;
class ServerTab;
class MainWindow : public QMainWindow
{
    Q_OBJECT


public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();


private:
    Ui::MainWindow *ui;
    LocalServerTab *m_localServerTab;
    QFC::Client *m_client;
    ServerTab *m_serverTab;
    QStackedLayout *m_layout;


private slots:
    void setupActions();
    void serverDialogFinished(const QString &server, const quint16 &port, const QString &nick);
    void clientConnected(QFC::Client *self);
    void newGame(QFC::Client *server, QFC::Game *game);
    void gameLeaved(QWidget *tab);
    void socketError(QFC::Client *self, const QAbstractSocket::SocketError &error, const QString &errorString);
    void gotErrorReply(QFC::Client *self, const QF::Protocol::Error &error);
    void connectToServerClicked();
    void showConnectToServerDialog();
    void startServerPrivateClicked();
    void startServerPrivate();
    void startServerPublicClicked();
    void startServerPublic();
    void serverTabCloseRequested(const int &index);
    void clientDisconnected(ServerTab *self);
    void configure();
    void showAbout();
    void fullScreenModeTriggered(const bool &enabled);
    void fullViewModeTriggered(GameTab *tab, const bool &enabled);


};


#endif // MAINWINDOW_H
