/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "playerlistdock.h"
#include "ui_playerlistdock.h"
#include "theme.h"

// Qt
#include <QtGui/QPixmap>


PlayerListDock::PlayerListDock(QWidget *parent)
    : QDockWidget(tr("Player List"), parent),
    ui(new Ui::PlayerListDock)
{

    ui->setupUi(this);

    connect(Theme::currentTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
    themeChanged();

}


PlayerListDock::~PlayerListDock()
{

    delete ui;

}


void PlayerListDock::addPlayer(const QString &nick, const int &id)
{

    QListWidgetItem *item = new QListWidgetItem;
    item->setText(nick);
    item->setData(Qt::UserRole+1, id);
    QPixmap icon(32, 32);
    icon.fill(Theme::currentTheme()->colorForRole((Theme::ColorRole) id));
    item->setIcon(QIcon(icon));
    ui->playerList->addItem(item);

}


void PlayerListDock::removePlayer(const QString &nick, const int &id)
{

    Q_UNUSED(nick);
    for (int i = 0; i < ui->playerList->count(); i++) {
        QListWidgetItem *item = ui->playerList->item(i);

        if (item->data(Qt::UserRole+1).toInt() == id) {
            ui->playerList->takeItem(i);
            delete item;
        }
    }

}


void PlayerListDock::setSelf(const int &id)
{

    m_self = id;

}


void PlayerListDock::setCurrentPlayer(const int &id)
{

    for (int i = 0; i < ui->playerList->count(); i++) {
        QListWidgetItem *item = ui->playerList->item(i);

        QFont font = item->font();
        font.setUnderline(item->data(Qt::UserRole+1).toInt() == id);
        item->setFont(font);
    }

}


void PlayerListDock::themeChanged()
{

    for (int i = 0; i < ui->playerList->count(); i++) {
        QListWidgetItem *item = ui->playerList->item(i);
        const int id = item->data(Qt::UserRole+1).toInt();

        item->setData(Qt::UserRole+1, id);
        QPixmap icon(32, 32);
        icon.fill(Theme::currentTheme()->colorForRole((Theme::ColorRole) id));
        item->setIcon(QIcon(icon));
    }

}

