/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef SERVERTAB_H
#define SERVERTAB_H

// own
#include "client.h"

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtGui/QWidget>


namespace Ui {
    class ServerTab;
}

class KeyDialog;
class ServerTab : public QWidget
{
    Q_OBJECT


public:
    explicit ServerTab(QWidget *parent = 0);
    ~ServerTab();

    bool isQuickGameServer() const;

    void setServer(QFC::Client *server);
    void startQuickGame(const QF::Layout &layout);

    QList<QWidget*> gameTabs() const;
    void addGameTab(QWidget *tab);
    void removeGameTab(QWidget *tab);


private:
    Ui::ServerTab *ui;
    QFC::Client *m_client;
    QList<QWidget*> m_gameTabs;


private slots:
    void gotGameList(const QHash<int, QString> &list);
    void gameOpen(const int &gameID, const QString &name);
    void gameClosed(const int &gameID, const QString &name);
    void joinGameClicked();
    void createGameClicked();
    void createGameDialogFinished(const QString &name, const QString &key, const QF::Layout &layout);
    void keyDialogFinished(KeyDialog *self);
    void clientDisconnected();
    void gameSelectionChanged();


signals:
    void disconnected(ServerTab *self);


};


#endif // SERVERTAB_H
