/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "servertabwidget.h"
#include "gametab.h"
#include "servertab.h"

// lib
#include <QFrustrationSharedGUI/global.h>


ServerTabWidget::ServerTabWidget(QWidget *parent)
    : QTabWidget(parent),
    m_server(0)
{

    setTabsClosable(true);
    m_serverTab = 0;

    connect(this, SIGNAL(tabCloseRequested(int)), this, SLOT(closeRequested(int)));

}


ServerTabWidget::~ServerTabWidget()
{



}


QFC::Client *ServerTabWidget::server() const
{

    return m_server;

}


void ServerTabWidget::setServer(QFC::Client *server)
{

    m_server = server;

    connect(server, SIGNAL(disconnected(QFC::Client*)), this, SLOT(disconnected(QFC::Client*)));
    connect(server, SIGNAL(gameCreated(QFC::Client*,QFC::Game*)), this, SLOT(newGame(QFC::Client*,QFC::Game*)));

    ServerTab *tab = new ServerTab(this);
    tab->setServer(server);
    addTab(tab, FRUST_ICON("network-server"), tr("Server"));
    setCurrentWidget(tab);

    m_serverTab = tab;

}


void ServerTabWidget::startQuickGame(const QF::Layout &layout, const int &kiCount)
{

    Q_ASSERT(m_serverTab);
    m_kiCount = kiCount;
    m_serverTab->startQuickGame(layout);

}


void ServerTabWidget::newGame(QFC::Client *server, QFC::Game *game)
{

    Q_UNUSED(server);

    GameTab *tab = new GameTab(this);
    connect(tab, SIGNAL(leaveGame(QWidget*)), this, SLOT(gameLeaved(QWidget*)));
    tab->setGame(game);
    addTab(tab, FRUST_ICON("applications-games"), game->name());
    setCurrentWidget(tab);

    if (m_serverTab->isQuickGameServer()) {
        tab->startQuickGame(m_kiCount);
    }

}


void ServerTabWidget::gameLeaved(QWidget *tab)
{

    removeTab(indexOf(tab));
    delete tab;

    if (m_serverTab->isQuickGameServer()) {
        emit quickGameFinished(this);
    }

}


void ServerTabWidget::disconnected(QFC::Client *)
{

    emit disconnected(this);

}


void ServerTabWidget::closeRequested(const int &index)
{

    QWidget *tab = widget(index);
    if (!tab || !m_serverTab || tab == m_serverTab) {
        return;
    }

    gameLeaved(tab);

}
