/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "soundcore.h"
#include "config.h"

// Qt
#include <QtCore/QVariant>
#include <QtCore/QUrl>
#ifdef HAVE_PHONON
    #include <phonon/MediaObject>
    #include <phonon/AudioOutput>
#endif


SoundCore::SoundCore(QObject *parent)
    : QObject(parent)
{

#ifdef HAVE_PHONON
    for (int i = 0; i < 3; i++) {
        Phonon::MediaObject *channel = Phonon::createPlayer(Phonon::GameCategory);
        m_channels << channel;

        connect(channel, SIGNAL(finished()), this, SLOT(channelFinished()));
    }
    m_freeChannels = m_channels;
#endif

    connect(Config::self(), SIGNAL(configChanged()), this, SLOT(configChanged()));
    configChanged();

}


SoundCore::~SoundCore()
{

#ifdef HAVE_PHONON
    qDeleteAll(m_channels);
#endif
}


void SoundCore::playFile(const QString &file)
{

#ifdef HAVE_PHONON
    Phonon::MediaObject *channel = getChannel();
    if (channel) {
        channel->setCurrentSource(Phonon::MediaSource(file));
        channel->play();
    } else {
        qDebug() << "no free channel....";
    }
#else
    Q_UNUSED(file);
#endif

}


void SoundCore::play(const QString &key)
{

#ifdef HAVE_PHONON
    if (!m_sounds.contains(key)) {
        qDebug() << "Sound not found:" << key;
        return;
    }

    if (m_enabled) {
        playFile(m_sounds.value(key));
    }
#else
    Q_UNUSED(key);
#endif

}


void SoundCore::addSound(const QString &key, const QString &file)
{

    m_sounds.insert(key, file);

}


#ifdef HAVE_PHONON
Phonon::MediaObject *SoundCore::getChannel()
{

    if (!m_freeChannels.isEmpty()) {
        return m_freeChannels.takeAt(0);
    }
    return 0;

}
#endif


void SoundCore::configChanged()
{

    m_enabled = Config::self()->value("Sound-Enabled").toBool();

}


void SoundCore::channelFinished()
{

#ifdef HAVE_PHONON
    Phonon::MediaObject *channel = static_cast<Phonon::MediaObject*>(sender());
    if (channel->state() == Phonon::ErrorState) {
        qWarning() << "Channel in error state..";
        m_channels.removeAll(channel);
        delete channel;

        channel = Phonon::createPlayer(Phonon::GameCategory);
        m_channels << channel;

        connect(channel, SIGNAL(finished()), this, SLOT(channelFinished()));
    }
    m_freeChannels.append(channel);
#endif

}

