/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef STONEWIDGET_H
#define STONEWIDGET_H

// lib
#include <QFrustrationLib/stone.h>

// Qt
#include <QtGui/QGraphicsWidget>
#include <QtCore/QPropertyAnimation>


class QSequentialAnimationGroup;
class StoneWidget : public QGraphicsWidget
{
    Q_OBJECT


public:
    explicit StoneWidget(QF::Stone *s, const QColor &color, const bool &isPlayerStone, QGraphicsWidget *parent = 0);
    ~StoneWidget();

    QF::Stone *stone() const;
    int possibleMove() const;
    bool isDown() const;
    bool isPlayerStone() const;

    void setPossibleMove(const int &move);
    void setDown(const bool &down);

    void stopMove();
    void animateMove(const QList<QRectF> &fields);


private:
    QF::Stone *m_stone;
    QColor m_color;
    int m_move;
    bool m_pressed;
    QPropertyAnimation *m_scaleAnimation;
    QSequentialAnimationGroup *m_moveGroup;
    bool m_isPlayerStone;


private slots:
    void moved(const int &from, const int &to);
    void moveFinished();


protected:
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);

    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);


signals:
    void stoneMoved(StoneWidget *self, const int &from, const int &to);
    void clicked(StoneWidget *self);


};


#endif // STONEWIDGET_H
