/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef THEME_H
#define THEME_H

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtCore/QObject>
#include <QtGui/QColor>
#include <QtCore/QHash>
#include <QtCore/QRectF>
#include <QtGui/QPixmap>


class QSvgRenderer;
class QPainter;
class Theme : public QObject
{
    Q_OBJECT
    friend class ThemeSingleton;


public:
    enum Margin {
        LeftMargin = 0,
        TopMargin = 1,
        RightMargin = 2,
        BottomMargin = 3
    };

    enum ColorRole {
        Player0Color = 0,
        Player1Color = 1,
        Player2Color = 2,
        Player3Color = 3,
        Player4Color = 4,
        Player5Color = 5
    };

    struct ThemeData {
        QString name;
        QString description;
        QString version;
        QString email;
        QString author;
        QString file;
        QString preview;

        QHash<Theme::ColorRole, QColor> colors;
    };

    ~Theme();

    static Theme *currentTheme();

    QColor colorForRole(const Theme::ColorRole &role) const;
    QList<Theme::ThemeData> availableThemes() const;
    QString currentThemeName() const;
    void setCurrentTheme(const QString &theme);

    void renderBackground(QPainter *painter, const QRectF &rect);
    void renderStone(const int &id, QPainter *painter, const QRectF &rect);
    void renderField(QPainter *painter, const QRectF &rect, const int &player, const QF::Layout::FieldType &type);
    void getBoardMargins(qreal *left, qreal *top, qreal *right, qreal *bottom, const QRectF &rect);
    void renderBoard(QPainter *painter, const QRectF &rect);
    void renderDice(QPainter *painter, const int &num, const QRectF &rect);


private:
    explicit Theme(QObject *parent = 0);

    ThemeData m_currentTheme;
    QSvgRenderer *m_renderer;

    qreal getMargin(const Theme::Margin &margin, const QString &element, const QRectF &rect) const;


signals:
    void themeChanged();


};

#endif // THEME_H
