/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef VIEW_H
#define VIEW_H


// lib
#include <QFrustrationLib/layout.h>
#include <QFrustrationLib/stone.h>

// Qt
#include <QtGui/QGraphicsView>
#include <QtCore/QHash>


class DiceWidget;
class StoneWidget;
class FieldWidget;
class View : public QGraphicsView
{
    Q_OBJECT


public:
    explicit View(QWidget *parent = 0);
    ~View();

    int currentZoom() const;

    void setLayout(const QF::Layout &layout);
    void addStone(QF::Stone *stone);
    void setTurn(const QHash<int, int> &turn);
    void setRoll(const int &roll);
    void setPlayerList(const QHash<int, QString> list);
    void setCurrentPlayer(const int &playerID);
    void setSelf(const int &id);
    void enableOpenGL(const bool &enable);


public slots:
    void setZoom(int percentZoom);


private:
    enum Direction {
        Left = 0,
        Up = 1,
        Right = 2,
        Down = 3
    };

    QStringList m_players;
    DiceWidget *m_dice;
    QHash<int, QString> m_playerList;
    int m_id;
    QHash<int, FieldWidget*> m_fields;
    QList<StoneWidget*> m_stones;
    QF::Layout m_layout;
    QTimer *m_resizeTimer;
    int m_fieldSize;
    QPixmap m_background;

    inline FieldWidget *getField(const int &id) const;
    inline StoneWidget *getStone(const int &id) const;
    inline int playerId(const QString &nick) const;
    inline void moveRect(const View::Direction &direction, QRectF *rect, const int &count, const int &fieldSize);
    inline QRectF topLeftFieldRect() const;
    inline StoneWidget *getCurrentStone() const;
    inline QRectF boardRect() const;
    inline int fieldSize() const;

    void reLayout();
    void scaleBy(qreal scaleFactor);


private slots:
    void resetWidgetStates(FieldWidget *skipField = 0, StoneWidget *skipStone = 0);
    void stoneMoved(StoneWidget *self, const int &from, const int &to);
    void fieldClicked(FieldWidget *field);
    void stoneClicked(StoneWidget *self);
    void delayedResize();
    void configChanged();


protected:
    void drawBackground(QPainter *painter, const QRectF &rect);
    void resizeEvent(QResizeEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);


signals:
    void sendTurn(const int &from, const int &to);
    void diceClicked();


};


#endif // VIEW_H
