/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "configtabwidget.h"
#include "configpage.h"

// lib
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtGui/QScrollBar>


ConfigTabWidget::ConfigTabWidget(QWidget *parent)
    : QListWidget(parent)
{

    setSelectionMode(QListWidget::SingleSelection);

    connect(this, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(tabClicked(QListWidgetItem*)));
    connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(tabSelectionChanged()));
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
    setIconSize(QSize(32, 32));

}


ConfigTabWidget::~ConfigTabWidget()
{


}


QSize ConfigTabWidget::sizeHint() const
{

    int width = sizeHintForColumn(0)+frameWidth()*2+5;
    if (verticalScrollBar()->isVisible()) {
        width += verticalScrollBar()->width();
    }

    return QSize(width, 100);

}


QList<ConfigPage*> ConfigTabWidget::tabs() const
{

    QList<ConfigPage*> list;
    for (int i = 0; i < count(); i++) {
        QListWidgetItem *item = QListWidget::item(i);
        list << (ConfigPage*) item->data(Qt::UserRole+1).value<void*>();
    }
    return list;

}


void ConfigTabWidget::addTab(ConfigPage *tab)
{

    QListWidgetItem *item = new QListWidgetItem;
    item->setText(tab->title());
    item->setIcon(FRUST_ICON(tab->icon()));
    item->setData(Qt::UserRole+1, qVariantFromValue((void*)tab));

    addItem(item);
    if (count() == 1) {
        tabClicked(item);
    }

}


void ConfigTabWidget::tabClicked(QListWidgetItem *item)
{

    item->setSelected(true);

}


void ConfigTabWidget::tabSelectionChanged()
{

    const QList<QListWidgetItem*> items = selectedItems();
    if (items.isEmpty()) {
        return;
    }
    ConfigPage *page = (ConfigPage*) items.at(0)->data(Qt::UserRole+1).value<void*>();
    emit tabRequested(page);

}
