/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "keybutton.h"

// lib
#include <QFrustrationSharedGUI/global.h>

// Qt
#include <QtGui/QKeyEvent>


KeyButton::KeyButton(QWidget *parent)
    : QPushButton(parent)
{

    m_currentKey = Qt::Key_unknown;
    connect(this, SIGNAL(clicked()), this, SLOT(start()));

}


KeyButton::~KeyButton()
{



}


Qt::Key KeyButton::key() const
{

    return m_currentKey;

}


void KeyButton::setKey(const Qt::Key &key)
{

    m_currentKey = key;
    setText(QKeySequence(key).toString());

}


void KeyButton::reset()
{

    setText(QKeySequence(m_currentKey).toString());

}


void KeyButton::start()
{

    setText(tr("Please press a key"));
    grabKeyboard();

}


void KeyButton::keyPressEvent(QKeyEvent *event)
{

    event->accept();

}


void KeyButton::keyReleaseEvent(QKeyEvent *event)
{

    event->accept();
    releaseKeyboard();

    emit keyRequested(this, (Qt::Key) event->key());

}
