/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// onw
#include "fieldwidget.h"

// Qt
#include <QtGui/QPainter>
#include <QtGui/QGraphicsSceneMouseEvent>


FieldWidget::FieldWidget(QGraphicsWidget *parent)
    : QGraphicsWidget(parent)
{

    m_type = QF::Layout::InvalidType;
    m_index = 1;
    m_playerID = 0;

}


int FieldWidget::playerID() const
{

    return m_playerID;

}


void FieldWidget::setPlayerID(int id)
{

    m_playerID = id;

}


int FieldWidget::index() const
{

    return m_index;

}


void FieldWidget::setIndex(int index)
{

    m_index = index;
    update();

}


QF::Layout::FieldType FieldWidget::fieldType() const
{

    return m_type;

}


void FieldWidget::setFieldType(const QF::Layout::FieldType &type)
{

    m_type = type;
    update();

}


void FieldWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{

    Q_UNUSED(option);
    Q_UNUSED(widget);

    QColor color;
    switch (playerID()) {
    case 0: color = Qt::red; break;
    case 1: color = Qt::blue; break;
    case 2: color = Qt::green; break;
    case 3: color = Qt::yellow; break;
    case 4: color = Qt::cyan; break;
    case 5: color = Qt::magenta; break;
    default: color = Qt::black; break;
    }

    switch (m_type) {
    case QF::Layout::WayType: color = Qt::white; break;
    case QF::Layout::StartType: color = color.lighter(); break;
    case QF::Layout::LastType: color = color.darker(); break;
    default: break;
    }


    painter->setRenderHints(QPainter::Antialiasing|QPainter::HighQualityAntialiasing|QPainter::TextAntialiasing);
    painter->setFont(font());

    QRadialGradient r(contentsRect().center(), contentsRect().height()/2, contentsRect().bottomRight());
    r.setColorAt(0, Qt::black);
    r.setColorAt(0.5, Qt::gray);
    r.setColorAt(1, Qt::white);


    painter->setBrush(QBrush(r));
    painter->drawEllipse(contentsRect());

    QRectF in = contentsRect().adjusted(4, 4, -4, -4);
    r = QRadialGradient(in.center(), in.height()/2, in.topLeft());
    r.setColorAt(0, color.darker());
    r.setColorAt(1, color);

    painter->setBrush(QBrush(r));
    painter->drawEllipse(in);

    if (false) {
        painter->setOpacity(0.6);
        painter->setBrush(QBrush(color, Qt::SolidPattern));
        painter->drawEllipse(contentsRect());
    }

    painter->drawText(contentsRect(), Qt::AlignCenter, QString::number(m_index));

}


void FieldWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();

}


void FieldWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    emit moved(this, event->scenePos());

}


void FieldWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    emit moveDone(this, event->scenePos());
    emit pressed(this);

}



