/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef MAPMAKERVIEW_H
#define MAPMAKERVIEW_H

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtGui/QGraphicsView>


class FieldWidget;
class MapMakerView : public QGraphicsView
{
    Q_OBJECT


public:
    enum EditMode {
        NormalMode = 0,
        WayMode = 1,
        AddMode = 2,
        RemoveMode = 3
    };

    explicit MapMakerView(QWidget *parent = 0);
    ~MapMakerView();

    bool isValid() const;

    void save(QByteArray *data);
    void load(const QByteArray &data);

    void setFields(const int &rows, const int &columns);
    void setEditMode(const MapMakerView::EditMode &mode);

    int players() const;
    int stones() const;
    int rows() const;
    int columns() const;


public slots:
    void addField();
    void test();

    void setPlayers(int players);
    void setStones(int stones);
    void setRows(int rows);
    void setColumns(int columns);

    void showCode();


private:
    int m_rows;
    int m_columns;
    int m_players;
    int m_stones;
    MapMakerView::EditMode m_mode;

    QRectF m_hoverRect;
    QList<QPointF> m_wayPoints;

    QList<QRectF> fieldRects() const;
    int getRow(FieldWidget *widget) const;
    int getColumn(FieldWidget *widget) const;
    QRectF getRect(int row, int column) const;
    FieldWidget *createField();
    QF::Layout buildLayout() const;

    void setFieldPos(QGraphicsWidget *widget, int row, int column);


private slots:
    void moved(FieldWidget *self, const QPointF &to);
    void moveDone(FieldWidget *self, const QPointF &to);


protected:
    void drawBackground(QPainter *painter, const QRectF &rect);
    void drawForeground(QPainter *painter, const QRectF &rect);

    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);


signals:
    void itemSelected(FieldWidget *field);
    void playersChanged(int players);
    void rowsChanged(int rows);
    void columnsChanged(int columns);
    void stonesChanged(int stones);
    void editModeChanged(const MapMakerView::EditMode &mode);


};


#endif // MAPMAKERVIEW_H
