/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "propertyeditor.h"
#include "ui_propertyeditor.h"
#include "fieldwidget.h"

// Qt
#include <QtCore/QMetaObject>
#include <QtCore/QMetaProperty>
#include <QtCore/QDebug>


PropertyEditor::PropertyEditor(QWidget *parent)
    : QWidget(parent)
{

    ui = new Ui::PropertyEditor;
    ui->setupUi(this);
    m_field = 0;

    ui->typeCombo->addItem(tr("Invalid field"), (int) QF::Layout::InvalidType);
    ui->typeCombo->addItem(tr("Start field"), (int) QF::Layout::StartType);
    ui->typeCombo->addItem(tr("Last field"), (int) QF::Layout::LastType);
    ui->typeCombo->addItem(tr("Target field"), (int) QF::Layout::TargetType);
    ui->typeCombo->addItem(tr("Home field"), (int) QF::Layout::HomeType);
    ui->typeCombo->addItem(tr("Way field"), (int) QF::Layout::WayType);

    connect(ui->typeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(currentTypeChanged(int)));

}


PropertyEditor::~PropertyEditor()
{

    delete ui;

}


void PropertyEditor::setField(FieldWidget *field)
{

    m_field = field;
    if (field) {
        ui->indexSpin->setValue(field->index());
        ui->playerSpin->setValue(field->playerID());

        for (int i = 0; i < ui->typeCombo->count(); i++) {
            if (ui->typeCombo->itemData(i).toInt() == (int) field->fieldType()) {
                ui->typeCombo->setCurrentIndex(i);
                break;
            }
        }
    }

}


void PropertyEditor::currentTypeChanged(const int &index)
{

    switch (ui->typeCombo->itemData(index).toInt()) {
    case QF::Layout::WayType: ui->playerSpin->setDisabled(true);
    default: ui->playerSpin->setEnabled(true); break;
    }

}


void PropertyEditor::save()
{

    if (!m_field) {
        return;
    }

    m_field->setPlayerID(ui->playerSpin->value());
    m_field->setIndex(ui->indexSpin->value());
    m_field->setFieldType((QF::Layout::FieldType)ui->typeCombo->itemData(ui->typeCombo->currentIndex()).toInt());
}
