/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef CLIENT_H
#define CLIENT_H

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtCore/QObject>


namespace QFS {


class Server;
class ClientPrivate;
class Client : public QObject
{
    Q_OBJECT
    friend class ClientPrivate;


public:
    explicit Client(QObject *parent, Server *server);
    ~Client();

    QString nick() const;
    bool init(const int &socket);

    void setNick(const QString &nick);
    void send(const QByteArray &blob);


private:
    ClientPrivate *d;


signals:
    void disconnected(QFS::Client *self);
    void connected(QFS::Client *self);
    void rollRequested(QFS::Client *self, const int &gameID);
    void turnRequested(QFS::Client *self, const int &from, const int &to, const int &gameID);
    void gameCreateRequested(QFS::Client *self, const QString &name, const QString &key, const QF::Layout &layout);
    void joinGameRequested(QFS::Client *self, const int &gameID, const QString &key);
    void startGameRequested(QFS::Client *self, const int &gameID);
    void gameListRequested(QFS::Client *self);
    void newGameChatText(QFS::Client *self, const QString &text, const QString &sender, const int &gameID);
    void leaveRequested(QFS::Client *self, const int &gameID);
    void receivingData(const qint64 &bytes);
    void dataSent(const qint64 &bytes);


};


} // namespace QFS


#endif // CLIENT_H
