/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef SERVER_GAME_H
#define SERVER_GAME_H

// lib
#include <QFrustrationLib/layout.h>
#include <QFrustrationLib/stone.h>

// Qt
#include <QtCore/QObject>
#include <QtCore/QHash>


namespace QFS {


class Client;
class Game : public QObject
{
    Q_OBJECT


public:
    explicit Game(QFS::Client *owner, const int &gameId, const QString &name, const QString &key,
                  const QF::Layout &layout,
                  QObject *parent = 0);
    ~Game();

    int gameId() const;
    QFS::Client *owner() const;
    QList<QFS::Client*> clients() const;
    QString name() const;
    QString key() const;
    QF::Layout layout() const;

    void startGame(QFS::Client *starter);

    void addPlayer(QFS::Client *client);
    void removePlayer(QFS::Client *client);
    void setMaximumPlayers(const int &max);

    void setTurn(QFS::Client *client, const int &from, const int &to);
    void createRoll(QFS::Client *client);

    void sendChat(const QString &sender, const QString &text);


private:
    Client *m_owner;
    int m_id;
    QHash<Client*, int> m_clients;
    int m_maxPlayers;
    QF::Layout m_layout;
    QList<QF::Stone*> m_stones;
    int m_currentPlayer;
    int m_rollCount;
    bool m_playerHasRolled;
    QHash<int, int> m_currentTurn;
    int m_currentRoll;
    bool m_started;
    QString m_name;
    QString m_key;

    void sendToClients(const QByteArray &blob);
    void nextTurn();

    int roll() const;
    QList<QF::Stone*> stonesInHome(const int &player) const;
    QF::Stone *stoneOnStart(const int &player) const;
    QList<QF::Stone*> stonesFromPlayer(const int &player) const;
    QF::Stone *stoneOnIndex(const int &index) const;
    bool hasStoneOnField(const int &player) const;
    bool hasHoleInTarget(const int &player) const;
    int getFreeHome(QF::Stone *stone) const;
    void createTurn();


private slots:
    void stoneMoved(const int &from, const int &to);
    void rollRequested(QFS::Client *client);
    void gotTurn(QFS::Client *self, const int &from, const int &to);


signals:
    void gameClosed(QFS::Game *self);
    void gameStarted(QFS::Game *self);


};


} // namespace QFS


#endif // SERVER_GAME_H
