/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "layout.h"
#include "layout_p.h"

// Qt
#include <QtCore/QDebug>


namespace QF {


LayoutPrivate::LayoutPrivate(Layout *parent)
    : q(parent)
{



}


LayoutPrivate::~LayoutPrivate()
{


}


Layout::Layout()
    : d(new LayoutPrivate(this))
{


}


Layout::Layout(const QF::Layout &other)
    : d(new LayoutPrivate(this))
{

    d->columns = other.d->columns;
    d->rows = other.d->rows;
    d->players = other.d->players;
    d->stones = other.d->stones;

    d->m_layout = other.d->m_layout;
    d->m_playerTargets = other.d->m_playerTargets;
    d->m_playerHomes = other.d->m_playerHomes;
    d->m_playerStarts = other.d->m_playerStarts;
    d->m_playerLasts = other.d->m_playerLasts;
    d->m_way = other.d->m_way;

}


Layout::~Layout()
{

    delete d;

}


QF::Layout &Layout::operator=(const QF::Layout &other)
{

    d->columns = other.d->columns;
    d->rows = other.d->rows;
    d->players = other.d->players;
    d->stones = other.d->stones;

    d->m_layout = other.d->m_layout;
    d->m_playerTargets = other.d->m_playerTargets;
    d->m_playerHomes = other.d->m_playerHomes;
    d->m_playerStarts = other.d->m_playerStarts;
    d->m_playerLasts = other.d->m_playerLasts;
    d->m_way = other.d->m_way;

    return *this;

}


QHash<int, QF::Layout::Field> Layout::getLayout() const
{

    return d->m_layout;

}


int Layout::players() const
{

    return d->players;

}


int Layout::stones() const
{

    return d->stones;

}


int Layout::rows() const
{

    return d->rows;

}


int Layout::columns() const
{

    return d->columns;

}


int Layout::maximumIndex() const
{

    return d->m_way.size();

}


void Layout::beginLayout(const int &rows, const int &columns, const int &players, const int &stones)
{

    d->rows = rows;
    d->columns = columns;
    d->players = players;
    d->stones = stones;

    d->m_layout.clear();
    d->m_playerTargets.clear();
    d->m_playerLasts.clear();
    d->m_playerStarts.clear();
    d->m_way.clear();

}


bool Layout::addField(const QF::Layout::FieldType &type, const int &index, const int &player, const int &row, const int &column)
{

    if (d->m_layout.contains(index)) {
        return false;
    }
    d->m_layout.insert(index, Field(type, row, column, player));
    return true;

}


static void insertList(QHash<int, QList<int> > *hash, int key, int value)
{

    QList<int> list = hash->value(key);
    list.append(value);
    hash->insert(key, list);

}


void Layout::endLayout()
{

    QHashIterator<int, Field> it(d->m_layout);
    while (it.hasNext()) {
        it.next();

        switch (it.value().type) {
        case StartType:
            d->m_way.insert(it.key(), it.value().type);
            d->m_playerStarts.insert(it.value().player, it.key());
            break;
        case LastType:
            d->m_way.insert(it.key(), it.value().type);
            d->m_playerLasts.insert(it.value().player, it.key());
            break;
        case TargetType: insertList(&d->m_playerTargets, it.value().player, it.key()); break;
        case HomeType: insertList(&d->m_playerHomes, it.value().player, it.key()); break;
        case WayType: d->m_way.insert(it.key(), it.value().type); break;
        case InvalidType: break;
        }
    }

}


void Layout::buildLayout(const QF::Layout::DefaultLayoutType &type)
{

    if (type == QF::Layout::FourPlayersLayout) {
        beginLayout(11, 11, 4, 4);

        // player0 home
        addField(QF::Layout::HomeType, 100, 0, 0, 0);
        addField(QF::Layout::HomeType, 101, 0, 0, 1);
        addField(QF::Layout::HomeType, 102, 0, 1, 0);
        addField(QF::Layout::HomeType, 103, 0, 1, 1);

        // player1 home
        addField(QF::Layout::HomeType, 104, 1, 0, 9);
        addField(QF::Layout::HomeType, 105, 1, 0, 10);
        addField(QF::Layout::HomeType, 106, 1, 1, 9);
        addField(QF::Layout::HomeType, 107, 1, 1, 10);

        // player2 home
        addField(QF::Layout::HomeType, 108, 2, 9, 9);
        addField(QF::Layout::HomeType, 109, 2, 9, 10);
        addField(QF::Layout::HomeType, 110, 2, 10, 9);
        addField(QF::Layout::HomeType, 111, 2, 10, 10);

        // player3 home
        addField(QF::Layout::HomeType, 112, 3, 9, 0);
        addField(QF::Layout::HomeType, 113, 3, 9, 1);
        addField(QF::Layout::HomeType, 114, 3, 10, 0);
        addField(QF::Layout::HomeType, 115, 3, 10, 1);

        // way
        addField(QF::Layout::StartType, 1, 0, 4, 0); // player0 start
        for (int i = 2; i < 6; i++) {
            addField(QF::Layout::WayType, i, -1, 4, i-1); // 4 field right
        }
        for (int i = 6; i < 10; i++) {
            addField(QF::Layout::WayType, i, -1, 9-i, 4); // 4 field up
        }
        addField(QF::Layout::LastType, 10, 1, 0, 5); // player1 last
        addField(QF::Layout::StartType, 11, 1, 0, 6); // player1 start
        for (int i = 12; i < 16; i++) { // 4 fields down
            addField(QF::Layout::WayType, i, -1, i-11, 6);
        }
        for (int i = 16; i < 20; i++) { // 4 fields right
            addField(QF::Layout::WayType, i, -1, 4, i-9);
        }
        addField(QF::Layout::LastType, 20, 2, 5, 10); // player2 last
        addField(QF::Layout::StartType, 21, 2, 6, 10); // player2 start
        for (int i = 22; i < 26; i++) { // 4 fields left
            addField(QF::Layout::WayType, i, -1, 6, 31-i);
        }
        for (int i = 26; i < 30; i++) { // 4 fields down
            addField(QF::Layout::WayType, i, -1, i-19, 6);
        }
        addField(QF::Layout::LastType, 30, 3, 10, 5); // player3 last
        addField(QF::Layout::StartType, 31, 3, 10, 4); // player3 start
        for (int i = 32; i < 36; i++) { // 4 fields up
            addField(QF::Layout::WayType, i, -1, 41-i, 4);
        }
        for (int i = 36; i < 40; i++) { // 4 fields left
            addField(QF::Layout::WayType, i, -1, 6, 39-i);
        }
        addField(QF::Layout::LastType, 40, 0, 5, 0); // player0 last

        for (int i = 41; i < 45; i++) { // player0 Target
            addField(QF::Layout::TargetType, i, 0, 5, i-40);
        }
        for (int i = 45; i < 49; i++) { // player1 Target
            addField(QF::Layout::TargetType, i, 1, i-44, 5);
        }
        for (int i = 49; i < 53; i++) { // player2 Target
            addField(QF::Layout::TargetType, i, 2, 5, 58-i);
        }
        for (int i = 53; i < 57; i++) { // player3 Target
            addField(QF::Layout::TargetType, i, 3, 62-i, 5);
        }
        endLayout();
    } else {
        // autgen with QFrustrationCreator
        beginLayout(16, 15, 6, 4);

        addField(QF::Layout::TargetType, 903, 2, 7, 9);
        addField(QF::Layout::TargetType, 902, 2, 6, 10);
        addField(QF::Layout::TargetType, 901, 2, 5, 11);
        addField(QF::Layout::TargetType, 900, 2, 4, 12);
        addField(QF::Layout::HomeType, 503, 4, 15, 0);
        addField(QF::Layout::HomeType, 502, 4, 15, 1);
        addField(QF::Layout::HomeType, 501, 4, 14, 1);
        addField(QF::Layout::HomeType, 500, 4, 14, 0);
        addField(QF::Layout::HomeType, 103, 0, 1, 0);
        addField(QF::Layout::HomeType, 102, 0, 1, 1);
        addField(QF::Layout::HomeType, 101, 0, 0, 1);
        addField(QF::Layout::HomeType, 100, 0, 0, 0);
        addField(QF::Layout::TargetType, 1003, 3, 8, 9);
        addField(QF::Layout::TargetType, 1002, 3, 9, 10);
        addField(QF::Layout::TargetType, 1001, 3, 10, 11);
        addField(QF::Layout::TargetType, 1000, 3, 11, 12);
        addField(QF::Layout::HomeType, 603, 5, 8, 0);
        addField(QF::Layout::HomeType, 602, 5, 8, 1);
        addField(QF::Layout::HomeType, 601, 5, 7, 1);
        addField(QF::Layout::HomeType, 600, 5, 7, 0);
        addField(QF::Layout::HomeType, 203, 1, 1, 13);
        addField(QF::Layout::HomeType, 202, 1, 1, 14);
        addField(QF::Layout::HomeType, 201, 1, 0, 14);
        addField(QF::Layout::HomeType, 200, 1, 0, 13);
        addField(QF::Layout::TargetType, 1103, 4, 11, 7);
        addField(QF::Layout::TargetType, 1102, 4, 12, 7);
        addField(QF::Layout::TargetType, 1101, 4, 13, 7);
        addField(QF::Layout::TargetType, 1100, 4, 14, 7);
        addField(QF::Layout::LastType, 50, 0, 3, 1);
        addField(QF::Layout::WayType, 49, 0, 4, 0);
        addField(QF::Layout::WayType, 48, 0, 5, 1);
        addField(QF::Layout::TargetType, 703, 0, 7, 5);
        addField(QF::Layout::WayType, 47, 0, 6, 2);
        addField(QF::Layout::TargetType, 702, 0, 6, 4);
        addField(QF::Layout::WayType, 46, 0, 7, 2);
        addField(QF::Layout::TargetType, 701, 0, 5, 3);
        addField(QF::Layout::WayType, 45, 0, 8, 2);
        addField(QF::Layout::TargetType, 700, 0, 4, 2);
        addField(QF::Layout::WayType, 44, 0, 9, 2);
        addField(QF::Layout::WayType, 43, 0, 10, 1);
        addField(QF::Layout::StartType, 42, 5, 11, 0);
        addField(QF::Layout::LastType, 41, 5, 12, 1);
        addField(QF::Layout::HomeType, 303, 2, 8, 13);
        addField(QF::Layout::WayType, 40, 0, 13, 2);
        addField(QF::Layout::HomeType, 302, 2, 8, 14);
        addField(QF::Layout::WayType, 39, 0, 12, 3);
        addField(QF::Layout::HomeType, 301, 2, 7, 14);
        addField(QF::Layout::WayType, 38, 0, 11, 4);
        addField(QF::Layout::HomeType, 300, 2, 7, 13);
        addField(QF::Layout::WayType, 37, 0, 12, 5);
        addField(QF::Layout::WayType, 36, 0, 13, 6);
        addField(QF::Layout::WayType, 35, 0, 14, 6);
        addField(QF::Layout::StartType, 34, 4, 15, 6);
        addField(QF::Layout::LastType, 33, 4, 15, 7);
        addField(QF::Layout::WayType, 32, 0, 15, 8);
        addField(QF::Layout::WayType, 31, 0, 14, 8);
        addField(QF::Layout::WayType, 30, 0, 13, 8);
        addField(QF::Layout::WayType, 29, 0, 12, 9);
        addField(QF::Layout::WayType, 28, 0, 11, 10);
        addField(QF::Layout::WayType, 27, 0, 12, 11);
        addField(QF::Layout::StartType, 26, 3, 13, 12);
        addField(QF::Layout::LastType, 25, 3, 12, 13);
        addField(QF::Layout::TargetType, 1203, 5, 8, 5);
        addField(QF::Layout::WayType, 24, 0, 11, 14);
        addField(QF::Layout::TargetType, 1202, 5, 9, 4);
        addField(QF::Layout::WayType, 23, 0, 10, 13);
        addField(QF::Layout::TargetType, 1201, 5, 10, 3);
        addField(QF::Layout::WayType, 22, 0, 9, 12);
        addField(QF::Layout::TargetType, 1200, 5, 11, 2);
        addField(QF::Layout::WayType, 21, 0, 8, 12);
        addField(QF::Layout::WayType, 20, 0, 7, 12);
        addField(QF::Layout::WayType, 19, 0, 6, 12);
        addField(QF::Layout::WayType, 18, 0, 5, 13);
        addField(QF::Layout::TargetType, 803, 1, 4, 7);
        addField(QF::Layout::StartType, 17, 2, 4, 14);
        addField(QF::Layout::TargetType, 802, 1, 3, 7);
        addField(QF::Layout::LastType, 16, 2, 3, 13);
        addField(QF::Layout::TargetType, 801, 1, 2, 7);
        addField(QF::Layout::WayType, 15, 0, 2, 12);
        addField(QF::Layout::TargetType, 800, 1, 1, 7);
        addField(QF::Layout::WayType, 14, 0, 3, 11);
        addField(QF::Layout::WayType, 13, 0, 4, 10);
        addField(QF::Layout::WayType, 12, 0, 3, 9);
        addField(QF::Layout::WayType, 11, 0, 2, 8);
        addField(QF::Layout::WayType, 10, 0, 1, 8);
        addField(QF::Layout::HomeType, 403, 3, 15, 13);
        addField(QF::Layout::StartType, 9, 1, 0, 8);
        addField(QF::Layout::HomeType, 402, 3, 15, 14);
        addField(QF::Layout::HomeType, 401, 3, 14, 14);
        addField(QF::Layout::LastType, 8, 1, 0, 7);
        addField(QF::Layout::HomeType, 400, 3, 14, 13);
        addField(QF::Layout::WayType, 7, 0, 0, 6);
        addField(QF::Layout::WayType, 6, 0, 1, 6);
        addField(QF::Layout::WayType, 5, 0, 2, 6);
        addField(QF::Layout::WayType, 4, 0, 3, 5);
        addField(QF::Layout::WayType, 3, 0, 4, 4);
        addField(QF::Layout::WayType, 2, 0, 3, 3);
        addField(QF::Layout::StartType, 1, 0, 2, 2);

        endLayout();
    }

}


void Layout::save(QByteArray *data) const
{

    QDataStream stream(data, QIODevice::WriteOnly);
    stream << QString("[Frust Layout]");
    stream << players();
    stream << stones();
    stream << rows();
    stream << columns();
    stream << d->m_layout.size();

    QHashIterator<int, Field> it(d->m_layout);
    while (it.hasNext()) {
        it.next();

        Field f = it.value();
        stream << it.key();
        stream << (int) f.type;
        stream << f.row;
        stream << f.column;
        stream << f.player;
    }

}


bool Layout::load(const QByteArray &data)
{

    QDataStream stream(data);
    QString head;
    stream >> head;

    if (head != QLatin1String("[Frust Layout]")) {
        qDebug() << "Invalid header" << head << stream.status();
        return false;
    }

    stream >> d->players;
    stream >> d->stones;
    stream >> d->rows;
    stream >> d->columns;

    int fieldCount;
    stream >> fieldCount;

    beginLayout(rows(), columns(), players(), stones());
    for (int i = 0; i < fieldCount; i++) {
        int index;
        int type;
        int row;
        int column;
        int player;

        stream >> index;
        stream >> type;
        stream >> row;
        stream >> column;
        stream >> player;

        addField((QF::Layout::FieldType) type, index, player, row, column);
    }
    endLayout();

    return true;

}


int Layout::tryMove(const int &player, const int &from, const int &count) const
{

//    qDebug() << "********** BEGIN TRY MOVE ************";

    const int maxIndex = d->m_way.keys().size();
    const int lastIndex = d->m_playerLasts.value(player);
    const int startIndex = d->m_playerStarts.value(player);
    const QList<int> Targets = d->m_playerTargets.value(player);

    int field = from+count;
    if (field > maxIndex && field < lastIndex) {
        field = field-maxIndex;
    }


   /* qDebug() << "TryMove" << "MaxIndex:" << maxIndex << "LastIndex:" << lastIndex <<
            "startIndex:" << startIndex << "Targets:" << Targets << "field:" << field <<
            "Count:" << count << "From:" << from << "player" << player;
*/


    // from one Target into another
    if (Targets.contains(from)) {
       // qDebug() << "Targets contains from";
        int TargetIndex = Targets.indexOf(from);
      //  qDebug() << "from index:" << TargetIndex;
        TargetIndex = TargetIndex+count;
      //  qDebug() << "to index" << TargetIndex << "Targets.size" << Targets.size();
        if (TargetIndex < Targets.size()) {
      //      qDebug() << "TargetIndex < Targets.size()";
        //    qDebug() << "********* END TRY MOVE    ************";
            return Targets.at(TargetIndex);
        } else {
         //   qDebug() << "********* END TRY MOVE    ************";
            return -1;
        }

    }


    // from way in Target
    if ((field > lastIndex && from < startIndex && startIndex > lastIndex) ||
        (field > lastIndex && from > startIndex && lastIndex > startIndex)) {
      //  qDebug() << "(field > lastIndex && from < startIndex && startIndex > lastIndex) || (field > lastIndex && from > startIndex && lastIndex > startIndex)";
        int TargetIndex = (field-lastIndex)-1;
       // qDebug() << "TargetIndex:" << TargetIndex << "Targets size:" << Targets.size();
        if (TargetIndex < Targets.size()) {
        //    qDebug() << "TargetIndex < Targets.size()";
       //     qDebug() << "********* END TRY MOVE    ************";
            return Targets.at(TargetIndex);
        } else {
       //     qDebug() << "********* END TRY MOVE    ************";
            return -1;
        }
    }

    // fix index
    if (field > maxIndex) {
       // qDebug() << "field > maxIndex";
        field = field-maxIndex;
     //   qDebug() << "new field " << field;
    }

    // normal
    if (d->m_layout.contains(field)) {
     //   qDebug() << "normal move";
     //   qDebug() << "********* END TRY MOVE    ************";
        return field;
    } else {
     //   qDebug() << "field:" << field << "not found....";
     //   qDebug() << "********* END TRY MOVE    ************";

    }

    return -1;

}


QList<int> Layout::getWay(const int &player, const int &from, const int &to) const
{

  //  qDebug() << "getWay" << player << from << to;

    const QList<int> home = homes(player);
    if (home.contains(from) || home.contains(to)) {
        return QList<int>() << from << to;
    }

    QList<int> list;
    int index = from;
    while (index != to) {
        index = tryMove(player, index, 1);
        Q_ASSERT(index != -1);
        list << index;
    }
    return list;

}


QList<int> Layout::homes(const int &player) const
{

    QList<int> list = d->m_playerHomes.value(player);
    qSort(list);
    return list;

}


QList<int> Layout::targets(const int &player) const
{

    QList<int> list = d->m_playerTargets.value(player);
    qSort(list);
    return list;

}


int Layout::startField(const int &player) const
{

    return d->m_playerStarts.value(player);

}


int Layout::lastField(const int &player) const
{

    return d->m_playerLasts.value(player);

}


bool Layout::isValid() const
{

    if (d->m_playerHomes.keys().size() != players()) {
        qDebug() << "ISVALID ERROR 1";
        return false;
    }

    if (d->m_playerTargets.keys().size() != players()) {
        qDebug() << "ISVALID ERROR 2";
        return false;
    }

    if (d->m_playerStarts.keys().size() != players()) {
        qDebug() << "ISVALID ERROR 3";
        return false;
    }

    if (d->m_playerLasts.keys().size() != players()) {
        qDebug() << "ISVALID ERROR 4";
        return false;
    }

    QHashIterator<int, QList<int> > homeIt(d->m_playerHomes);
    while (homeIt.hasNext()) {
        homeIt.next();

        if (homeIt.value().size() != stones()) {
            qDebug() << "ISVALID ERROR 5";
            return false;
        }
    }

    QHashIterator<int, QList<int> > TargetIt(d->m_playerTargets);
    while (TargetIt.hasNext()) {
        TargetIt.next();

        if (TargetIt.value().size() != stones()) {
            qDebug() << "ISVALID ERROR 6";
            return false;
        }
    }

    for (int i = 1; i < d->m_way.keys().size(); i++) {
        if (!d->m_way.contains(i)) {
            qDebug() << "ISVALID ERROR 7"  << i;
            return false;
        }
    }

    return true;

}



} // namespace QF
