/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef LAYOUT_H
#define LAYOUT_H


// own
#include "QFrustrationLib_global.h"

// Qt
#include <QtCore/QObject>
#include <QtCore/QList>
#include <QtCore/QHash>


namespace QF {


class LayoutPrivate;
class QFRUSTRATIONLIBSHARED_EXPORT Layout
{
    Q_ENUMS(FieldType)


public:
    enum FieldType {
        InvalidType = -1,
        WayType = 0,
        HomeType = 1,
        TargetType = 2,
        StartType = 3,
        LastType = 4
    };

    enum DefaultLayoutType {
        FourPlayersLayout = 0,
        SixPlayersLayout = 1
    };

    class QFRUSTRATIONLIBSHARED_EXPORT Field {
    public:
        Field(QF::Layout::FieldType t, int r, int c, int p)
        { type = t; row = r; column = c; player = p; }
        Field(){}
        ~Field(){}
        QF::Layout::FieldType type;
        int row;
        int column;
        int player;
    };

    explicit Layout();
    Layout(const QF::Layout &other);
    ~Layout();

    QF::Layout &operator=(const QF::Layout &other);

    QHash<int, QF::Layout::Field> getLayout() const;
    int players() const;
    int stones() const;
    int rows() const;
    int columns() const;
    int maximumIndex() const;

    void beginLayout(const int &rows, const int &columns, const int &players, const int &stones);
    bool addField(const QF::Layout::FieldType &type, const int &index, const int &player, const int &row, const int &column);
    void endLayout();
    void buildLayout(const QF::Layout::DefaultLayoutType &type);
    void save(QByteArray *data) const;
    bool load(const QByteArray &data);

    int tryMove(const int &player, const int &from, const int &count) const;
    QList<int> getWay(const int &player, const int &from, const int &to) const;
    QList<int> homes(const int &player) const;
    QList<int> targets(const int &player) const;
    int startField(const int &player) const;
    int lastField(const int &player) const;
    bool isValid() const;


private:
    LayoutPrivate *d;


};


} // namespace QF


#endif // LAYOUT_H
