/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef SERVER_H
#define SERVER_H

// own
#include "QFrustrationLib_global.h"

// Qt
#include <QtCore/QObject>


namespace QFS {


class Client;
class ServerPrivate;
class QFRUSTRATIONLIBSHARED_EXPORT Server : public QObject
{
    Q_OBJECT
    friend class ServerPrivate;


public:
    explicit Server(QObject *parent = 0);
    ~Server();

    quint16 port() const;
    QString hostName() const;
    QStringList players() const;
    bool isNickValid(const QString &nick);
    bool start(const bool &publicServer);
    QString address() const;
    QString errorString() const;

    void setPort(const quint16 &port);


private:
    ServerPrivate *d;


signals:
    void clientConnected(const QString &nick);
    void clientDisconnected(const QString &nick);
    void gameCreated(const QString &name);
    void gameStarted(const QString &name);
    void gameClosed(const QString &name);
    void receivingData(const qint64 &bytes);
    void dataSent(const qint64 &bytes);


};


} // namespace QFS


#endif // SERVER_H
