/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef SERVER_P_H
#define SERVER_P_H


// lib
#include <QFrustrationLib/stone.h>

// lib
#include <QFrustrationLib/layout.h>

// Qt
#include <QtNetwork/QTcpServer>


namespace QFS {

class Game;
class Client;
class Server;
class ServerPrivate : public QTcpServer
{
    Q_OBJECT


public:
    ServerPrivate(Server *parent);
    ~ServerPrivate();

    quint16 port;
    //int maximumPlayers;
    QList<Client*> clients;
    QList<QFS::Game*> m_games;

    void sendToAllClients(const QByteArray &blob);


private:
    Server *q;


private slots:
    void clientConnected(QFS::Client *self);
    void clientDisconnected(QFS::Client *client);
    void clientRollRequested(QFS::Client *self, const int &gameID);
    void clientTurnRequested(QFS::Client *self, const int &from, const int &to, const int &gameID);
    void gameCreateRequested(QFS::Client *self, const QString &name, const QString &key, const QF::Layout &layout);
    void joinGameRequested(QFS::Client *self, const int &gameID, const QString &key);
    void startGameRequested(QFS::Client *self, const int &gameID);
    void gameListRequested(QFS::Client *self);
    void gameClosed(QFS::Game *self);
    void newGameChatText(QFS::Client *self, const QString &text, const QString &sender, const int &gameID);
    void leaveRequested(QFS::Client *self, const int &gameID);
    void gameStarted(QFS::Game *self);


protected:
    void incomingConnection(int socketDescriptor);


};


}


#endif // SERVER_P_H
