/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
//#include "server.h"
#include <QFrustrationLib/server.h>

// Qt
#include <QtCore/QCoreApplication>
#include <QtCore/QStringList>
#include <QtCore/QObject>
#include <QtCore/QDebug>

// C
#include <stdio.h>

#define QL1S(str) QLatin1String(str)

#define MAX_PLAYERS 6
#define DEFAULT_PORT 8080


void printError(const QString &msg)
{

    fprintf(stderr, qPrintable(QString("<%1> %2\n").arg("ERROR").arg(msg)));
    exit(1);

}


void printMessage(const QString &msg)
{

    fprintf(stdout, qPrintable(QString(msg).append("\n")));

}


void checkNextArg(const QStringList &args, int &pos)
{

    pos++;
    if (pos >= args.size()) {
        printError(QObject::tr("Missing argument!"));
    }

}



int main(int argc, char *argv[])
{

    QCoreApplication app(argc, argv);

    bool publicServer = false;
    quint16 port = DEFAULT_PORT;

    const QStringList args = app.arguments();
    for (int i = 0; i < args.size(); i++) {
        const QString arg = args.at(i);

        if (arg == QL1S("--public")) {
            publicServer = true;
        } else if (arg == QL1S("--port")) {
            checkNextArg(args, i);

            bool ok;
            port = args.at(i).toUShort(&ok);
            if (!ok) {
                printError(QObject::tr("Invalid port!"));
            }
        }
    }

    printMessage(QObject::tr("Settings:"));
    printMessage(QObject::tr("Port:").append(QString(" %1").arg(port)));
    printMessage(QObject::tr("Maximum Players: %1").arg(MAX_PLAYERS));
    printMessage(QObject::tr("Public Server: %1").arg(publicServer ? "true" : "false"));
    printMessage(QString());
    printMessage(QObject::tr("Starting Server..."));

    QFS::Server server;
    server.setPort(port);

    if (server.start(publicServer)) {
        printMessage(QObject::tr("Server started!"));
    } else {
        printError(QObject::tr("Could not start server!"));
    }

    return app.exec();

}
