/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "aboutdialog.h"
#include "ui_aboutdialog.h"
#include "aboutdialog_p.h"
#include "global.h"

// Qt
#include <QtGui/QTableWidgetItem>
#include <QtGui/QDesktopServices>


AboutDialogPrivate::AboutDialogPrivate(AboutDialog *parent)
    : QObject(parent),
      ui(new Ui::AboutDialog),
      q(parent)
{

    ui->setupUi(q);
    ui->iconLabel->setPixmap(FRUST_ICON("frust").pixmap(64, 64));
    connect(ui->donateButton, SIGNAL(clicked()), this, SLOT(donate()));

}


AboutDialogPrivate::~AboutDialogPrivate()
{

    delete ui;

}


void AboutDialogPrivate::donate()
{

    QDesktopServices::openUrl(QUrl(donateUrl));

}


AboutDialog::AboutDialog(QWidget *parent)
    : QDialog(parent),
    d(new AboutDialogPrivate(this))
{

    setAttribute(Qt::WA_DeleteOnClose, true);

    d->ui->closeButton->setIcon(FRUST_ICON("dialog-close"));
    d->ui->appNameLabel->setText(QString("<b>%1<b>").arg(qApp->applicationName()));
    d->ui->appVersionLabel->setText(qApp->applicationVersion());
    d->ui->licenseLabel->setText("<a href=\"http://www.gnu.org/licenses/gpl-2.0.html\">GPLV2</a>");

    setWindowIcon(qApp->windowIcon());
    setWindowTitle(tr("About %1").arg(qApp->applicationName()));

}


AboutDialog::~AboutDialog()
{

    delete d;

}


void AboutDialog::showAbout(QWidget *parent, const QHash<QString, QString> &devs, const QUrl &donateUrl)
{

    AboutDialog *dialog = new AboutDialog(parent);
    dialog->d->ui->table->setRowCount(devs.size());
    int r = 0;
    QHashIterator<QString, QString> it(devs);
    while (it.hasNext()) {
        it.next();

        QTableWidgetItem *name = new QTableWidgetItem;
        name->setText(it.key());

        QTableWidgetItem *contact = new QTableWidgetItem;
        contact->setText(it.value());

        dialog->d->ui->table->setItem(r, 0, name);
        dialog->d->ui->table->setItem(r, 1, contact);
        ++r;
    }

    dialog->d->ui->table->horizontalHeader()->resizeSections(QHeaderView::ResizeToContents);

    if (donateUrl.isEmpty()) {
        dialog->d->ui->donateButton->hide();
    } else {
        dialog->d->donateUrl = donateUrl;
    }

    dialog->show();
    dialog->raise();

}

