/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "errordialog.h"
#include "errordialog_p.h"
#include "global.h"

// Qt
#include <QtCore/QCoreApplication>


ErrorDialogPrivate::ErrorDialogPrivate(ErrorDialog *parent)
    : QObject(parent),
      q(parent)
{

}


ErrorDialogPrivate::~ErrorDialogPrivate()
{


}



static QHash<QString, ErrorDialog*> s_dialogs;
ErrorDialog::ErrorDialog(QWidget *parent)
    : QMessageBox(parent),
    d(new ErrorDialogPrivate(this))
{

    setAttribute(Qt::WA_DeleteOnClose, true);
    setIcon(QMessageBox::Critical);
    setStandardButtons(QMessageBox::Close);

}


ErrorDialog::~ErrorDialog()
{

    s_dialogs.remove(s_dialogs.key(this));
    delete d;

}


void ErrorDialog::showError(QWidget *parent, const QString &title, const QString &text)
{

    // show only one dialog
    if (s_dialogs.contains(title+text)) {
        return;
    }

    ErrorDialog *dialog = new ErrorDialog(parent);
    dialog->setWindowTitle(QString("%1: %2").arg(qApp->applicationName()).arg(title));
    dialog->setText(text);

    s_dialogs.insert(title+text, dialog);

    dialog->show();
    dialog->raise();

}
