/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "filerequester.h"
#include "filerequester_p.h"
#include "ui_filerequester.h"
#include "global.h"

// Qt
#include <QtGui/QFileDialog>


FileRequesterPrivate::FileRequesterPrivate(FileRequester *parent)
    : QObject(parent),
      q(parent)
{

    ui = new Ui::FileRequester;
    ui->setupUi(q);
    ui->button->setIcon(FRUST_ICON("folder-open"));

    connect(ui->button, SIGNAL(clicked()), this, SLOT(buttonClicked()));

}


FileRequesterPrivate::~FileRequesterPrivate()
{

    delete ui;

}


void FileRequesterPrivate::buttonClicked()
{

    QFileDialog *dialog = new QFileDialog(q);
    dialog->setDirectory(q->file());
    dialog->setFileMode(QFileDialog::AnyFile);
    dialog->setNameFilters(filters);
    dialog->setAttribute(Qt::WA_DeleteOnClose, true);
    connect(dialog, SIGNAL(fileSelected(QString)), this, SLOT(fileSelected(QString)));

    dialog->show();
    dialog->raise();

}



void FileRequesterPrivate::fileSelected(const QString &file)
{

    ui->edit->setText(file);

}



FileRequester::FileRequester(QWidget *parent)
    : QWidget(parent),
      d(new FileRequesterPrivate(this))
{

    connect(d->ui->edit, SIGNAL(textChanged(QString)), this, SIGNAL(fileChanged(QString)));
    d->ui->edit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);

}


FileRequester::~FileRequester()
{

    delete d;

}


QString FileRequester::file() const
{

    return d->ui->edit->text();

}


void FileRequester::setFile(const QString &file)
{

    d->ui->edit->setText(file);

}


void FileRequester::setNameFilters(const QStringList &filters)
{

    d->filters = filters;

}

