/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "global.h"

// Qt
#include <QtCore/QFileInfo>
#include <QtCore/QDir>
#include <QtCore/QDebug>
#include <QtCore/QCoreApplication>
#include <QtCore/QFile>


FrustGlobal::FrustGlobal()
{


}


class FrustGlobalInstance
{
public:
    FrustGlobal i;
};
FRUSTRATION_SINGLETON(FrustGlobalInstance, global_private_instance)


FrustGlobal *FrustGlobal::instance()
{

    return &global_private_instance->i;

}


QString FrustGlobal::findFrustDir(const QString &dir)
{

    if (m_cached_Dirs.contains(dir)) {
        return m_cached_Dirs.value(dir);
    }

    QString result;

#ifdef Q_OS_UNIX
    const QString app = QCoreApplication::organizationName();
    QString str = qgetenv("XDG_DATA_DIRS");
    if (str.isEmpty()) {
        str = "/usr/local/share/:/usr/share/";
    }
    QStringList dataDirs = str.split(':');
    for (int i = 0; i < dataDirs.size(); i++) {
        QDir d(dataDirs.at(i)+'/'+app);
        //qDebug() << "dir:" << dataDirs.at(i)+'/'+app << "app:" << app;
        if (d.cd(dir)) {
            result = d.absolutePath();
            break;
        }
    }
#endif

#ifdef Q_OS_WIN32
    QString str = QCoreApplication::applicationDirPath();
    QDir d(str);
    if (d.cd(dir)) {
        result = d.absolutePath();
    }
#endif
    m_cached_Dirs.insert(dir, result);
    return result;

}


QIcon FrustGlobal::fustIcon(const QString &name)
{

    if (!QIcon::hasThemeIcon(name)) {
        const QFileInfo info(findFrustDir("icons")+QDir::separator()+name+".png");
        if (info.exists()) {
            return QIcon(info.absoluteFilePath());
        }
        qDebug() << "no such icon:" << name;
        return QIcon();
    }
    return QIcon::fromTheme(name, QIcon(QString(":/icons/%1.png").arg(name)));

}
