/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef GLOBAL_H
#define GLOBAL_H


// own
#include "export.h"

// Qt
#include <QtCore/QString>
#include <QtGui/QIcon>
#include <QtCore/QHash>



#define FRUSTRATION_SINGLETON(TYPE, NAME) \
typedef void (*JCleanUpFunction)(); \
class JCleanUpGlobalStatic \
{ \
public: \
    JCleanUpFunction func; \
    inline ~JCleanUpGlobalStatic() { func(); } \
}; \
static QBasicAtomicPointer<TYPE> _j_static_##NAME = Q_BASIC_ATOMIC_INITIALIZER(0); \
static bool _j_static_##NAME##_destroyed;                                          \
static struct NAME {                                                               \
    inline bool isDestroyed() const                                                \
    {                                                                              \
        return _j_static_##NAME##_destroyed;                                       \
    }                                                                              \
    inline bool exists() const                                                     \
    {                                                                              \
        return _j_static_##NAME != 0;                                              \
    }                                                                              \
    inline operator TYPE*()                                                        \
    {                                                                              \
        return operator->();                                                       \
    }                                                                              \
    inline TYPE *operator->()                                                      \
    {                                                                              \
        if (!_j_static_##NAME) {                                                   \
            if (isDestroyed()) {                                                   \
                qFatal("Fatal Error: Accessed global static '%s *%s()' after destruction. " \
                       "Defined at %s:%d", #TYPE, #NAME, __FILE__, __LINE__);               \
            }                                                                               \
            TYPE *x = new TYPE;                                                             \
            if (!_j_static_##NAME.testAndSetOrdered(0, x) && _j_static_##NAME != x ) {      \
                delete x;                                                                   \
            } else {                                                                        \
                static JCleanUpGlobalStatic cleanUpObject = { destroy };                    \
            }                                                                               \
        }                                                                                   \
        return _j_static_##NAME;                                                            \
    }                                                                                       \
    inline TYPE &operator*()                                                                \
    {                                                                                       \
        return *operator->();                                                               \
    }                                                                                       \
    static void destroy()                                                                   \
    {                                                                                       \
        _j_static_##NAME##_destroyed = true;                                                \
        TYPE *x = _j_static_##NAME;                                                         \
        _j_static_##NAME = 0;                                                               \
        delete x;                                                                           \
    }                                                                                       \
} NAME;


class QFRUSTRATIONSHAREDGUILIBSHARED_EXPORT FrustGlobal
{
    friend class  FrustGlobalInstance;


public:
    static FrustGlobal *instance();

    QString findFrustDir(const QString &dir);
    QIcon fustIcon(const QString &name);


private:
    FrustGlobal();

    QHash<QString, QString> m_cached_Dirs;


};



#define FRUST_ICON(name) FrustGlobal::instance()->fustIcon(name)


#endif // GLOBAL_H
