/***************************************************************************
 *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "questiondialog.h"
#include "questiondialog_p.h"
#include "global.h"

// Qt
#include <QtGui/QApplication>


QuestionDialogPrivate::QuestionDialogPrivate(QuestionDialog *parent)
    : QObject(parent),
      q(parent)
{

    connect(q, SIGNAL(buttonClicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));

}


QuestionDialogPrivate::~QuestionDialogPrivate()
{


}


void QuestionDialogPrivate::buttonClicked(QAbstractButton *button)
{

    switch (q->standardButton(button)) {
    case QMessageBox::Yes: emit q->yesClicked(); break;
    case QMessageBox::No: emit q->noClicked(); break;
    default: break;
    }

}



QuestionDialog::QuestionDialog(QWidget *parent)
    : QMessageBox(parent),
      d(new QuestionDialogPrivate(this))
{

    setAttribute(Qt::WA_DeleteOnClose, true);
    setIcon(QMessageBox::Question);
    setStandardButtons(QMessageBox::Yes|QMessageBox::No);

}


QuestionDialog::~QuestionDialog()
{

    delete d;

}



QuestionDialog *QuestionDialog::showQuestion(QWidget *parent, const QString &title, const QString &question)
{

    QuestionDialog *dialog = new QuestionDialog(parent);
    dialog->setWindowTitle(QString("%1: %2").arg(qApp->applicationName()).arg(title));
    dialog->setText(question);

    dialog->show();
    dialog->raise();

    return dialog;

}

